 /************************************************************************
 *       Copyright (C) 1993, by WATCOM International Corp. All rights    *
 *       reserved. No part of this software may be reproduced            *
 *       in any form or by any means - graphic, electronic or            *
 *       mechanical, including photocopying, recording, taping           *
 *       or information storage and retrieval systems - except           *
 *       with the written permission of WATCOM International Corp.       *
 *************************************************************************/

#if !defined( _DBTOOLS_H_INCLUDED )
#define _DBTOOLS_H_INCLUDED

#include "sqlca.h"
#include "dllapi.h"

//***************************************************************************
//  Version number
//***************************************************************************

#define DB_TOOLS_VERSION_NUMBER	    6

// Changes in this version requiring a version number change:
//  - a_change_log:
//	- added truncation point and generation number options

#define DB_TOOLS_VERSION_5_0_03     5

// Changes in this version requiring a version number change:
//  - a_backup:
//	- added "hotlog_filename" to allow hot transaction log backup


#define DB_TOOLS_VERSION_3_2_D	    4

// Changes in this version requiring a version number change:
//  - a_compress_db:
//	- added "confirmrtn" and "noconfirm" to allow confirmation of
//	  replacing output file
//  - an_expand_db:
//	- added "confirmrtn" and "noconfirm" to allow confirmation of
//	  replacing output file

#define DB_TOOLS_VERSION_3_2_C	    3

// Changes in this version requiring a version number change:
//  - a_compress_db:
//	- extended structure to fill in compression statistics to avoid
//	  writing them directly.
//  - a_db_info:
//	- extended structure to allow returning of page usage information
//        from DBInfo.

//***************************************************************************
//  Generic interface pieces
//***************************************************************************

typedef struct a_name {
    struct a_name _fd_ *    next;
    char		    name[1];
} a_name, _fd_ * p_name;

typedef struct a_sysinfo {
    char	    valid_data		: 1;	// are following values set?
    char	    blank_padding	: 1;
    char	    case_sensitivity	: 1;
    char	    encryption		: 1;
    char	    default_collation[11];
    unsigned short  page_size;
} a_sysinfo;

//***************************************************************************
// initialization (DBTINIT) interface
//***************************************************************************
typedef struct a_dbtools_info {
    MSG_CALLBACK	errorrtn;
} a_dbtools_info;

extern _crtn short _entry DBToolsInit( const a_dbtools_info _fd_ * );
extern _crtn short _entry DBToolsFini( const a_dbtools_info _fd_ * );


//***************************************************************************
// get shared library version
//***************************************************************************

extern _crtn short _entry DBToolsVersion( void );

//***************************************************************************
// backup database (DBBACKUP) interface
//***************************************************************************

typedef struct a_backup_db {
    unsigned short	version;
    const char _fd_	*output_dir;
    const char _fd_	*connectparms;	// userid,password,database
    const char _fd_	*startline;		// (NULL for default start line)
    MSG_CALLBACK	confirmrtn;
    MSG_CALLBACK	errorrtn;
    MSG_CALLBACK	msgrtn;
    MSG_CALLBACK	statusrtn;
    char		backup_database	: 1;
    char		backup_logfile	: 1;
    char		backup_writefile: 1;	// only if using write file
    char		no_confirm	: 1;
    char		quiet		: 1;
    char		rename_log	: 1;
    char		truncate_log	: 1;
    const char _fd_	*hotlog_filename;
} a_backup_db;

extern _crtn short _entry DBBackup( const a_backup_db _fd_ * );

//***************************************************************************
// change database logfile name (DBLOG) interface
//***************************************************************************

typedef struct a_change_log {
    unsigned short	version;
    const char _fd_ *	dbname;
    const char _fd_ *	logname;	// (logname = NULL) for no log
    MSG_CALLBACK	errorrtn;
    MSG_CALLBACK	msgrtn;
    char		query_only		: 1;
    char		quiet			: 1;
    char		mirrorname_present	: 1;
    char		change_mirrorname	: 1;
    char		change_logname		: 1;
    char		ignore_ltm_trunc	: 1;
    char		ignore_remote_trunc	: 1;
    char		set_generation_number	: 1;

    // Following fields are new with 5.0:
    const char _fd_ *	mirrorname;
    unsigned short	generation_number;
} a_change_log;

extern _crtn short _entry DBChangeLogName( const a_change_log _fd_ * );

//***************************************************************************
// (1) change write file to refer to another database file (DBWRITE -d)
// (2) create a database write file (DBWRITE -c)
// (3) get status of a write file (DBWRITE -s)
//***************************************************************************

typedef struct a_writefile {
    unsigned short	version;
    const char _fd_ *	writename;
    const char _fd_ *	wlogname;	// "create" only
    const char _fd_ *	dbname;		// "change", "create"
    const char _fd_ *	forcename;	// forced filename reference
    MSG_CALLBACK	confirmrtn;	// "create" only
    MSG_CALLBACK	errorrtn;
    MSG_CALLBACK	msgrtn;
    char		action;		// used internally
    char		quiet   		: 1;
    char		erase   		: 1;	// "create" only; erase without confirm
    char		force 			: 1;
    char		mirrorname_present	: 1;

    // new for v5.0
    const char _fd_ *	wlogmirrorname;	// "create" only
} a_writefile;

extern _crtn short _entry DBChangeWriteFile( const a_writefile _fd_ * );
extern _crtn short _entry DBCreateWriteFile( const a_writefile _fd_ * );
extern _crtn short _entry DBStatusWriteFile( const a_writefile _fd_ * );

//***************************************************************************
// compress a database (DBSHRINK) interface
//***************************************************************************

typedef struct a_stats_line {
    long		pages;
    long		bytes;
    long		compressed_bytes;
} a_stats_line;

typedef struct a_compress_stats {
    a_stats_line	tables;
    a_stats_line	indices;
    a_stats_line	other;
    a_stats_line	free;
    a_stats_line	total;
    long		free_pages;
    long		unchanged;
} a_compress_stats;

typedef struct a_compress_db {
    unsigned short	version;
    const char _fd_ *	dbname;
    const char _fd_ *	compress_name;
    MSG_CALLBACK	errorrtn;
    MSG_CALLBACK	msgrtn;
    MSG_CALLBACK	statusrtn;
    char		display_free_pages	: 1;
    char		quiet			: 1;
    char		record_unchanged	: 1;

    // The following values are new with version 3.2c:

    a_compress_stats _fd_ * stats;	// filled in if not NULL
					// && display_free_pages not zero

    // The following values are new with version 3.2d:

    MSG_CALLBACK	confirmrtn;
    char		noconfirm		: 1;
} a_compress_db;

extern _crtn short _entry DBCompress( const a_compress_db _fd_ * );

//***************************************************************************
// create a new database (DBINIT) interface
//***************************************************************************

enum {			// verbosity
    VB_QUIET,
    VB_NORMAL,
    VB_VERBOSE
};

enum {			// Possible values for blank_pad
    NO_BLANK_PADDING,
    BLANK_PADDING
};

typedef struct a_create_db {
    unsigned short	version;
    const char _fd_ *	dbname;
    const char _fd_ *	logname;			// (NULL for no log file)
    const char _fd_ *	startline;			// (NULL for default start line)
    short		page_size;
    const char _fd_ *	default_collation;		// collation identifier
    MSG_CALLBACK	errorrtn;
    MSG_CALLBACK	msgrtn;
    short		database_version;		// used internally
    char		verbose;			// VB_xxx (above)
    char		blank_pad		: 2;
    char		respect_case    	: 1;
    char		encrypt			: 1;
    char		debug			: 1; 	// should be FALSE
    char		dbo_avail		: 1;
    char		mirrorname_present	: 1;

    // The following fields are used internally by DBCreate.

    short       	collation_id;

    // Following fields are new with 5.0:

    const char _fd_ *	dbo_username;
    const char _fd_ *	mirrorname;
} a_create_db;

extern _crtn short _entry DBCreate( a_create_db _fd_ * );

//***************************************************************************
// erase a database (DBERASE)
//***************************************************************************

typedef struct an_erase_db {
    unsigned short	version;
    const char _fd_ *	dbname;
    MSG_CALLBACK	confirmrtn;
    MSG_CALLBACK	errorrtn;
    MSG_CALLBACK	msgrtn;
    char		quiet : 1;
    char		erase : 1;	// erase without confirmation
} an_erase_db;

extern _crtn short _entry DBErase( const an_erase_db _fd_ * );

//***************************************************************************
// expand a compressed database (DBEXPAND) interface
//***************************************************************************

typedef struct an_expand_db {
    unsigned short	version;
    const char _fd_ *	compress_name;
    const char _fd_ *	dbname;
    MSG_CALLBACK	errorrtn;
    MSG_CALLBACK	msgrtn;
    MSG_CALLBACK	statusrtn;
    char		quiet : 1;

    // The following values are new with version 3.2d:

    MSG_CALLBACK	confirmrtn;
    char		noconfirm		: 1;
} an_expand_db;

extern _crtn short _entry DBExpand( const an_expand_db _fd_ * );

//***************************************************************************
// get information from a database about how it was created (DBINFO)
//***************************************************************************

typedef struct a_table_info {
    struct a_table_info _fd_ * next;
    unsigned short  table_id;	    // table id
    unsigned long   table_pages;    // number of table pages
    unsigned long   index_pages;    // number of index pages
    unsigned long   table_used;	    // bytes used in table pages
    unsigned long   index_used;	    // bytes used in index pages
    char _fd_ *	    table_name;	    // table name
} a_table_info;

typedef struct a_db_info {
    unsigned short  version;
    MSG_CALLBACK    errorrtn;
    const char _fd_ *dbname;		// file to extract info from
    unsigned short  dbbufsize;		// size of "dbnamebuffer"
    char _fd_ *	    dbnamebuffer;	// filled in
    unsigned short  logbufsize;		// size of "lognamebuffer"
    char _fd_ *	    lognamebuffer;	// filled in; empty if no log
    unsigned short  wrtbufsize;		// size of "wrtnamebuffer"
    char _fd_ *	    wrtnamebuffer;	// filled in; empty if not write file
    char	    quiet : 1;
    char	    mirrorname_present : 1;

    // The following values are returned by DBInfo:

    a_sysinfo	    sysinfo;
    unsigned long   free_pages;
    unsigned char   compressed : 1;

    // The following values are new with version 3.2c:

    const char _fd_ *connectparms;	// userid,password,database
    const char _fd_ *startline;		// (NULL for default start line)
    MSG_CALLBACK    msgrtn;
    MSG_CALLBACK    statusrtn;
    unsigned char   page_usage : 1;	// fill in page usage values below?

    // The following page usage values are returned by DBInfo:

    a_table_info _fd_ * totals;		// page usage statistics
    unsigned long   file_size;		// file size (in pages)
    unsigned long   unused_pages;	// number of unused (free) pages
    unsigned long   other_pages;	// number of other pages

    // New for v5.0
    unsigned short  mirrorbufsize;	// size of "lognamebuffer"
    char _fd_ *	    mirrornamebuffer;	// filled in; empty if no log
} a_db_info;

extern _crtn short _entry DBInfo( a_db_info _fd_ * );
extern _crtn short _entry DBInfoDump( a_db_info _fd_ * );
extern _crtn short _entry DBInfoFree( a_db_info _fd_ * );

//***************************************************************************
// get collation information from database (DBCOLLAT)
//***************************************************************************

typedef struct a_db_collation {
    unsigned short	version;
    const char _fd_ *	connectparms;	// userid,password,database
    const char _fd_ *	startline;	// (NULL for default start line)
    const char _fd_ *	collation_label;
    const char _fd_ *	filename;
    MSG_CALLBACK	confirmrtn;
    MSG_CALLBACK	errorrtn;
    MSG_CALLBACK	msgrtn;
    char		include_empty	: 1;// include lines with no mappings?
    char		hex_for_extended: 1;// use hex for extended chars (> 7E)?
    char		replace		: 1;// replace file w/o confirm?
    char		quiet		: 1;
} a_db_collation;

extern _crtn short _entry DBCollate( const a_db_collation _fd_ * );

//***************************************************************************
// translate log file to SQL (DBTRAN) interface
//***************************************************************************
    
typedef enum dbtran_userlist_type {
    DBTRAN_INCLUDE_ALL,
    DBTRAN_INCLUDE_SOME,
    DBTRAN_EXCLUDE_SOME
} dbtran_userlist_type;

typedef struct a_translate_log {
    unsigned short	version;
    const char _fd_ *	logname;
    const char _fd_ *	sqlname;
    p_name		userlist;
    MSG_CALLBACK	confirmrtn;
    MSG_CALLBACK	errorrtn;
    MSG_CALLBACK	msgrtn;
    char		userlisttype;		// DBTRAN_xxx (above)
    char		remove_rollback : 1;
    char		ansi_sql	: 1;
    char		since_checkpoint: 1;
    char		omit_comments   : 1;
    char		replace		: 1;	// replace SQL file w/o confirm?
    char		debug		: 1;
    char		include_trigger_trans : 1;
    char		comment_trigger_trans : 1;
} a_translate_log;

extern _crtn short _entry DBTranslateLog( const a_translate_log _fd_ * );

//***************************************************************************
// unload the database into files (DBUNLOAD)
//***************************************************************************

enum {
    UNLOAD_ALL,
    UNLOAD_DATA_ONLY,
    UNLOAD_NO_DATA
};

typedef struct an_unload_db {
    unsigned short	version;
    const char _fd_ *	connectparms;	// userid,password,database
    const char _fd_ *	startline;	// (NULL for default start line)
    const char _fd_ *	temp_dir;
    const char _fd_ *	reload_filename;// (Must be filled in)
    MSG_CALLBACK	errorrtn;
    MSG_CALLBACK	msgrtn;
    MSG_CALLBACK	statusrtn;
    MSG_CALLBACK	confirmrtn;
    char		unload_type;	// UNLOAD_xxx (above)
    char		verbose;	// VB_xxx (above)
    char		unordered : 1;
    char		no_confirm : 1; 
    char		use_internal_unload : 1;
    char		dbo_avail : 1;
    char		extract : 1;
    char		table_list_provided : 1;
    char		exclude_tables : 1;
    char		more_flag_bits_present : 1;
    // ** NOTE ** Do not add any more flags here!!

    // The following values are only written to by DBUnload:

    a_sysinfo	    sysinfo;

    const char _fd_ *	remote_dir;

    // Following fields added in 5.0:

    const char _fd_ *	dbo_username;

    // Fields for dbextract (added in 5.0)

    const char _fd_ *	subscriber_username;
    const char _fd_ *	publisher_address_type;
    const char _fd_ *	publisher_address;
    unsigned short	isolation_level;
    char		start_subscriptions : 1;
    char		exclude_foreign_keys : 1;
    char		exclude_procedures : 1;
    char		exclude_triggers : 1;
    char		exclude_views : 1;
    char		isolation_set : 1;
    char		include_where_subscribe : 1;
    
    p_name		table_list;	    // selective table list
    
    // Extended bits -- these are present if 'more_flag_bits_available' is true
    unsigned short	escape_char_present : 1;
    unsigned short	view_iterations_present : 1;
    
    unsigned short	view_iterations;
    char		escape_char;
} an_unload_db;

extern _crtn short _entry DBUnload( an_unload_db _fd_ * );

//***************************************************************************
// upgrade the system tables of a database (DBUPGRAD)
//***************************************************************************

typedef struct an_upgrade_db {
    unsigned short	version;
    const char _fd_ *	connectparms;	// userid,password,database
    const char _fd_ *	startline;	// (NULL for default start line)
    MSG_CALLBACK	errorrtn;
    MSG_CALLBACK	msgrtn;
    MSG_CALLBACK	statusrtn;
    char		quiet : 1;
    char		dbo_avail : 1;
    const char _fd_ *	dbo_username;
} an_upgrade_db;

extern _crtn short _entry DBUpgrade( const an_upgrade_db _fd_ * );

//***************************************************************************
// validate the tables of a database (DBVALID)
//***************************************************************************

typedef struct a_validate_db {
    unsigned short	version;
    const char _fd_ *	connectparms;	// userid,password,database
    const char _fd_ *	startline;	// (NULL for default start line)
    p_name		tables;
    MSG_CALLBACK	errorrtn;
    MSG_CALLBACK	msgrtn;
    MSG_CALLBACK	statusrtn;
    char		quiet : 1;
} a_validate_db;

extern _crtn short _entry DBValidate( const a_validate_db _fd_ * );

//***************************************************************************
// truncate transaction log interface
//***************************************************************************

typedef struct a_truncate_log {
    unsigned short	version;
    const char _fd_ *	connectparms;	// userid,password,database
    const char _fd_ *	startline;	// (NULL for default start line)
    MSG_CALLBACK	errorrtn;
    MSG_CALLBACK	msgrtn;
    char		quiet	: 1;
} a_truncate_log;

extern _crtn short _entry DBTruncateLog( const a_truncate_log _fd_ * );

#endif
