/*
 * Copyright (c) 1980 Regents of the University of California.
 * All rights reserved.  The Berkeley software License Agreement
 * specifies the terms and conditions for redistribution.
 *
 * @(#)netdb.h 5.7 (Berkeley) 5/12/86
 */

#ifndef _NETDB_H_INCLUDED

/****************************************************************************
*
* (C) Unpublished Copyright Novell, Inc. All Rights Reserved.
*
* No part of this file may be duplicated, revised, translated,
* localized or modified in any manner or compiled, linked or 
* uploaded or downloaded to or from any computer system without
* the prior written consent of Novell, Inc.
*
* NetWare C NLM Runtime Library source code
*
*****************************************************************************/


/*
 * $Header:   I:/proj/nwnfs3/rel3.0/tools/sdk40.b3/novh40/vcs/netdb.ovg   1.0   07 Jun 1993 16:03:50   lcheong  $
 * 
 * $Abstract:
 * Standard definitions for accessing the socket interface's network
 * database in Novell's NetWare 386 TCP/IP.  Since process context is
 * limited in NetWare 386, we need to play some games to provide context
 * to the database.
 * $
 * 
 * $Implementation Notes:
 * The actual NetWare 386 TCP/IP routines take an additional parameter
 * to provide them a block for maintaining context.  The normal routines
 * are actually macros which call the context aware routines.
 * 
 * One modification is required for porting an NLM the NetWare 386
 * versions of the database routines: a context block must be defined.
 * This is done by using the macro NETDB_DEFINE_CONTEXT in any one
 * module linked into the NLM.
 * 
 * If the preprocessor symbol NOT_NETWARE_386 is defined, this becomes
 * the standard netdb.h from 4.3BSD for use in more typical environments.
 * $
 */

/* The HOSTS database macros (i.e. gethostxxx) have the capability to
 *  evaluate either to the routines that access just the local /etc/hosts
 *  file (i.e. NWgethostxxx), or else the routines that automatically access
 *  a combination of local file, DNS, and NIS (i.e. NetDBgethostxxx).  The
 *  former case is the way previous SDK usage of the macro was implemented.
 *  The latter case is a newer option that utilizes network name services
 *  transparent to the NLM, but it requires NETDB.NLM (which is also provided
 *  in this SDK).  NETDB.NLM is an extension to TCP/IP and may be freely
 *  distributed with your product.
 *
 *  The developer may choose which routines to use by directly calling the
 *  routines desired (either NWgethostxxx or NetDBgethostxxx).  If the macros
 *  are used, then the macros will call the local-file-only versions of the
 *  calls (i.e. NWgethostxxx) unless the symbol NETDB_USE_INTERNET is
 *  defined below.  If you wish to use the internet name services such as
 *  NIS or DNS in addition to the local hosts file for host access, then this
 *  symbol must be defined (either here or in your source file).
 */
/* #define NETDB_USE_INTERNET  1 */

/*
 * Structures returned by network
 * data base library.  All addresses
 * are supplied in host order, and
 * returned in network order (suitable
 * for use in system calls).
 */
struct   hostent {
   char  *h_name; /* official name of host */
   char  **h_aliases;   /* alias list */
   int   h_addrtype; /* host address type */
   int   h_length;   /* length of address */
   char  **h_addr_list; /* list of addresses from name server */
#define  h_addr   h_addr_list[0] /* address, for backward compatiblity */
};

/*
 * Assumption here is that a network number
 * fits in 32 bits -- probably a poor one.
 */
struct   netent {
   char     *n_name; /* official name of net */
   char     **n_aliases;   /* alias list */
   int      n_addrtype; /* net address type */
   unsigned long  n_net;      /* network # */
   unsigned long  n_mask;     /* net mask -- Novell extension */
};

struct   servent {
   char  *s_name; /* official service name */
   char  **s_aliases;   /* alias list */
   int   s_port;     /* port # */
   char  *s_proto;   /* protocol to use */
};

struct   protoent {
   char  *p_name; /* official protocol name */
   char  **p_aliases;   /* alias list */
   int   p_proto; /* protocol # */
};


#ifdef NOT_NETWARE_386
/* Provide normal definitions if a compiler switch tells us that this
 * isn't NetWare.
 */

/* Don't need to define a context block in a normal environment. */
#define  NETDB_DEFINE_CONTEXT

#ifdef __cplusplus
extern "C" {
#endif
      
struct hostent *gethostbyname(), *gethostbyaddr(), *gethostent();
struct netent  *getnetbyname(), *getnetbyaddr(), *getnetent();
struct servent *getservbyname(), *getservbyport(), *getservent();
struct protoent   *getprotobyname(), *getprotobynumber(), *getprotoent();

#ifdef __cplusplus
}
#endif

/* Normal h_errno is just a simple int. */
extern  int h_errno; 

#else /* #ifdef NOT_NETWARE_386 */
/* But normally we assume this is NetWare 386, so we have to fake
 * the whole thing up to provide context.
 */

/*TODO: These are defined for now but will probably change:Andy*/
#define  SCRATCHBUFSIZE 1024
#define MAXALIASES   10
#define MAXALIASSIZE 64
#define MAXNAMESIZE  64
#define MAXADDRSIZE  19
#define MAXHOSTADDR  1

/* Special Novell structure for providing context in the otherwise
 * context-free NetWare 386 environment. The applications SHOULD NOT
 * access this structure !.
 */
typedef struct nwsockent {
   FILE           *nse_hostctx;
   FILE           *nse_netctx;
   FILE           *nse_protoctx;
   FILE           *nse_servctx;
   int      nse_h_errno;
   union sockent {
      struct hostent  nsu_hst;
      struct netent   nsu_net;
      struct protoent nsu_proto;
      struct servent  nsu_serv;
   }               nse_sockent_un;
   char            nse_scratch[SCRATCHBUFSIZE];
} nwsockent;

/* Declare the context block.  The client must supply the actual
 * block by placing NETDB_DEFINE_CONTEXT in one of the C modules
 * in the link.
 */
extern struct nwsockent nwSocketCtx;

/* Macro to define the actual context block somewhere. */
#define  NETDB_DEFINE_CONTEXT struct nwsockent nwSocketCtx = {0};

#ifdef __cplusplus
extern "C" {
#endif
      
/* ------------------------------------------------------------------------
                        Host file examination
   ------------------------------------------------------------------------ */

/* Local-file-only routines */

extern struct hostent *NWgethostbyname( struct nwsockent *nwsktctx,
               char      *name
                  );
extern struct hostent *NWgethostbyaddr( struct nwsockent *nwsktctx,
               char      *addr,
               int        length,
               int        type
                  );
extern struct hostent *NWgethostent( struct nwsockent *nwsktctx );
extern void        NWsethostent( struct nwsockent *nwsktctx,
                 int        stayopen
               );
extern void        NWendhostent( struct nwsockent *nwsktctx );

/* Internet Name Service routines */

/*************************************************************************
 * NetDBgethostbyname() -- returns the host entry (struct hostent *) given
 *  the name of a host.
 *
 *  The local file sys:/etc/hosts is consulted first to see if the entry
 *  exists there.  If so, then that is returned.  If not, then if DNS is
 *  installed on the machine, it will be consulted to perform the lookup.
 *  If the host still is not found, then NIS will be consulted if at all
 *  possible.
 *
 *  This function returns NULL when an error occurs.  The integer
 *  nwsktent->nse_h_errno can be checked to determine the nature of the
 *  error.
 *
 *  The integer nwsktent->nse_h_errno can have the following values:
 *
 *    HOST_NOT_FOUND       No such host exists.
 *
 *  If the NetDBgethostbyname function succeeds, it will return a pointer
 *  to a structure of type struct hostent.
 *
 *  Syntax:
 *    struct hostent * NetDBgethostbyname(struct nwsockent *nwsktent,
 *                                        char *name);
 *
 *       nwskent: Points to a context block.
 *
 *       name:    Official name of the host.
 *
 *  Returns:
 *    A pointer to the appropriate struct hostent if any that matches.
 *    NULL if no match found.
 *************************************************************************/
extern struct hostent * NetDBgethostbyname( struct nwsockent *nwskent,
               char        *name
                  );

/*************************************************************************
 * NetDBgethostbyaddr() -- returns the host entry (struct hostent *) given
 *  the address of a host.
 *
 *  The local file sys:/etc/hosts is consulted first to see if the entry
 *  exists there.  If so, then that is returned.  If not, then if DNS is
 *  installed on the machine, it will be consulted to perform the lookup.
 *  If the host still is not found, then NIS will be consulted if at all
 *  possible.
 *
 *  This function returns NULL when an error occurs.  The integer
 *  nwsktent->nse_h_errno can be checked to determine the nature of the
 *  error.
 *
 *  The integer nwsktent->nse_h_errno can have the following values:
 *
 *    HOST_NOT_FOUND       No such host exists.
 *
 *  If the NetDBgethostbyaddr function succeeds, it will return a pointer
 *  to a structure of type struct hostent.
 *
 *  Syntax:
 *    struct hostent * NetDBgethostbyaddr(struct nwsockent *nwskent,
 *                                        char *addr, int len, int type);
 *
 *       nwsktent: (Input) Points to a context block.
 *
 *       addr:     (Input) Internet address of the host.
 *
 *       len:      (Input) Length of the Internet address, in bytes.
 *
 *       type:     (Input) Value corresponding to the type of Internet
 *                 address.  Currently, the type is always AF_INET.
 *
 *  Returns:
 *    A pointer to the appropriate struct hostent if any that matches.
 *    NULL if no match found.
 *************************************************************************/
extern struct hostent * NetDBgethostbyaddr(struct nwsockent *nwsktent,
               char       *addr,
               int         len,
               int         type
                  );
             
/*************************************************************************
 * NetDBgethostent() -- returns the next sequential entry from the
 *  SYS:ETC/HOSTS file, opening the file it it is not already open.  Once
 *  the local file is depleted, all of the NIS host entries will be
 *  retrieved until those are depleted.
 *
 *  Note that there may be duplicate entries in the local and NIS databases.
 *  The caller should handle these appropriately.
 *
 *  This function returns NULL when an error occurs.  The integer
 *  nwsktent->nse_h_errno can be checked to determine the nature of the
 *  error.
 *
 *  The integer nwsktent->nse_h_errno can have the following values:
 *
 *    HOST_NOT_FOUND       No more hosts exist in either SYS:ETC/HOSTS or
 *                         NIS.
 *
 *  Syntax:
 *    struct hostent * NetDBgethostent(struct nwsockent *nwsktent,
 *                                     short * ploc);
 *
 *    nwsktent:  (Input) Points to a context block.
 *
 *    ploc:      (Output) If non-NULL, this short will indicate if this
 *               entry is from the local sys:etc/hosts file (NETDB_LOC_LOCAL)
 *               or from the NIS database (NETDB_LOC_NIS).
 *
 *               Pass in NULL if you're not interested in this information.
 *
 *  Returns:
 *    A pointer to the next host entry if the function is successful.
 *    NULL if no more entries or an error occurred.
 *************************************************************************/
extern struct hostent * NetDBgethostent( struct nwsockent *nwsktent,
               short       *ploc
                  );

/*************************************************************************
 * NetDBsethostent() -- rewinds the SYS:ETC/HOSTS file if the file is
 *  already open.  This call guarantees that the next call to
 *  NetDBgethostent() will return the FIRST record in the local hosts file,
 *  regardless of whether the LAST call returned an entry from the local
 *  file or from NIS.
 *
 *  If the stayopen flag is set (nonzero), the SYS:ETC/HOSTS file is NOT
 *  closed after each call made to NetDBgethostbyname() or
 *  NetDBgethostbyaddr().
 *
 *  Syntax:
 *    void NetDBsethostent(struct nwsockent *nwsktent, int stayopen);
 *
 *    nwsktent:  (Input) Points to a context block.
 *
 *    stayopen:  (Input) If nonzero, causes SYS:ETC/HOSTS to remain open
 *               after a call to NetDBgethostbyname() or
 *               NetDBgethostbyaddr().
 *
 *  Returns:
 *    Nothing.
 *************************************************************************/
extern void NetDBsethostent( struct nwsockent *nwsktent,
                   int      stayopen
                      );

/*************************************************************************
 * NetDBendhostent() -- closes the SYS:ETC/HOSTS file.  Also ends access
 *  to the NIS database.  After this call, the next call to
 *  NetDBgethostent() will be from the beginning of the local file again.
 *
 *  Syntax:
 *    void NetDBendhostent(struct nwsockent *nwsktent);
 *
 *    nwsktent:  (Input) Points to a context block.
 *
 *  Returns:
 *    Nothing.
 *************************************************************************/
extern void        NetDBendhostent( struct nwsockent *nwsktent );

/*************************************************************************
 * NetDBgethostname() -- this gets the current machine's host name into the
 *  passed in buffer (if it is large enough).
 *
 *  This will use the local hosts file if it exists, otherwise it will then
 *  try both DNS and NIS if available in order to get the official name of
 *  our own machine.
 *
 *  Syntax:
 *    int NetDBgethostname(struct nwsockent *nwsktent, char *name,
 *                                                          int namelen);
 *
 *    nwsktent: (Input)  Points to a context block.
 *
 *    name:     (Output) Official name of the host.
 *
 *    namelen:  (Input)  Specifies the size of the array pointed to by name.
 *
 *  Returns:
 *     0: The call succeeded.
 *    -1: The call failed.
 *************************************************************************/
extern int NetDBgethostname( struct nwsockent *nwsktent,
                   char        *name,
                   int          namelen
                      );

/* ------------------------------------------------------------------------ */

/* Network file examination */
extern struct netent *NWgetnetbyname( struct nwsockent *nwsktctx,
                  char         *name
                );
extern struct netent *NWgetnetbyaddr( struct nwsockent *nwsktctx,
                  long     net,
                  int      type
                );
extern struct netent *NWgetnetent( struct nwsockent *nwsktctx );
extern void       NWsetnetent( struct nwsockent *nwsktctx,
               int           stayopen
             );
extern void       NWendnetent( struct nwsockent *nwsktctx );

/* Service file examination */
extern struct servent *NWgetservbyname( struct nwsockent *nwsktctx,
               char      *name,
               char      *protocol
                  );
extern struct servent *NWgetservbyport( struct nwsockent *nwsktctx,
               int        port,
               char      *protocol
                  );
extern struct servent *NWgetservent( struct nwsockent *nwsktctx );
extern void        NWsetservent( struct nwsockent *nwsktctx,
                 int        stayopen
               );
extern void        NWendservent( struct nwsockent *nwsktctx );

/* Protocol file examination */
extern struct protoent *NWgetprotobyname( struct nwsockent *nwsktctx,
                 char         *name
               );
extern struct protoent *NWgetprotobynumber( struct nwsockent *nwsktctx,
                   int           protocol
                 );
extern struct protoent *NWgetprotoent( struct nwsockent *nwsktctx );
extern void    NWsetprotoent( struct nwsockent *nwsktctx,
                   int      stayopen
                 );
extern void    NWendprotoent( struct nwsockent *nwsktctx );
extern int     gethostname(char *name,int namelen);
extern long    gethostid(void);

#ifdef __cplusplus
}
#endif

/* Macros mapping the standard 4.3BSD names to calls with context
 * to our versions of the routines.
 */

#ifdef NETDB_USE_INTERNET

#define  gethostbyname( name )            \
   NetDBgethostbyname( &nwSocketCtx, (name) )
#define  gethostbyaddr( addr, length, type )    \
   NetDBgethostbyaddr( &nwSocketCtx, (addr), (length), (type) )
#define  gethostent()               \
   NetDBgethostent( &nwSocketCtx , NULL )
#define  sethostent( stayopenflag )       \
   NetDBsethostent( &nwSocketCtx, (stayopenflag) )
#define  endhostent()               \
   NetDBendhostent( &nwSocketCtx )
      
#else  // !NETDB_USE_INTERNET

#define  gethostbyname( name )            \
   NWgethostbyname( &nwSocketCtx, (name) )
#define  gethostbyaddr( addr, length, type )    \
   NWgethostbyaddr( &nwSocketCtx, (addr), (length), (type) )
#define  gethostent()               \
   NWgethostent( &nwSocketCtx )
#define  sethostent( stayopenflag )       \
   NWsethostent( &nwSocketCtx, (stayopenflag) )
#define  endhostent()               \
   NWendhostent( &nwSocketCtx )
      
#endif  // NETDB_USE_INTERNET

#define  getnetbyname( name )          \
   NWgetnetbyname( &nwSocketCtx, (name) )
#define  getnetbyaddr( net, type )        \
   NWgetnetbyaddr( &nwSocketCtx, (net), (type) )
#define  getnetent()             \
   NWgetnetent( &nwSocketCtx )
#define  setnetent( stayopenflag )        \
   NWsetnetent( &nwSocketCtx, (stayopenflag) )
#define  endnetent()             \
   NWendnetent( &nwSocketCtx )
#define  getservbyname( name, protocol )        \
   NWgetservbyname( &nwSocketCtx, (name), (protocol) )
#define  getservbyport( port, protocol )        \
   NWgetservbyport( &nwSocketCtx, (port), (protocol) )
#define  getservent()               \
   NWgetservent( &nwSocketCtx )
#define  setservent( stayopenflag )       \
   NWsetservent( &nwSocketCtx, (stayopenflag) )
#define  endservent()               \
   NWendservent( &nwSocketCtx )
#define  getprotobyname( name )           \
   NWgetprotobyname( &nwSocketCtx, (name) )
#define  getprotobynumber( protocol )        \
   NWgetprotobynumber( &nwSocketCtx, (protocol) )
#define  getprotoent()              \
   NWgetprotoent( &nwSocketCtx )
#define  setprotoent( stayopenflag )         \
   NWsetprotoent( &nwSocketCtx, (stayopenflag) )
#define  endprotoent()              \
   NWendprotoent( &nwSocketCtx )

/* h_errno is also context dependent, so define a macro for it, too. */
#define  h_errno  nwSocketCtx.nse_h_errno

#endif   /* end of #else clause of #ifdef NOT_NETWARE_386 */

/*
 * Error return codes from gethostbyname() and gethostbyaddr()
 */

#define  HOST_NOT_FOUND 1 /* Authoritive Answer Host not found */
#define  TRY_AGAIN   2 /* Non-Authoritive Host not found, or SERVERFAIL */
#define  NO_RECOVERY 3 /* Non recoverable errors, FORMERR, REFUSED, NOTIMP */
#define NO_ADDRESS   4 /* Valid host name, no address, look for MX record */


#define _NETDB_H_INCLUDED
#endif  /* _NETDB_H_INCLUDED  */
