/*

----------------------------------------------------------------------------

	(C) Copyright 1993 Novell, Inc.
	All Rights Reserved.

	This program is an unpublished copyrighted work which is proprietary
	to Novell, Inc. and contains confidential information that is not
	to be reproduced or disclosed to any other person or entity without
	prior written consent from Novell, Inc. in each and every instance.

	WARNING:  Unauthorized reproduction of this program as well as
	unauthorized preparation of derivative works based upon the
	program or distribution of copies by sale, rental, lease or
	lending are violations of federal copyright laws and state trade
	secret laws, punishable by civil and criminal penalties.

----------------------------------------------------------------------------
*/


#ifndef _SA_GENER_H_
#define _SA_GENER_H_

/* Constants */
/* --------- */

/* maximum number of elements in any object we're supporting.  If
 * the generic support comes across a request for a longer object,
 * it truncates the request to this size.  Notice that this is the
 * number of elements, not the number bytes, which makes a difference
 * for object IDs.
 */
#define	SAG_MAX_ELEMENTS	256


/* Macros */
/* ------ */

/* define access to an object organized as a simple array of SNMP values. */
#define	SAI_DEFINE_TABLE( Object_Name, Table_Location )			\
    extern ObjControl Object_Name = {					\
			OCBF_NONE,	/* no flags */			\
			Table_Location,	/* pointer to table */		\
			SAGSimpleGet,	/* our get handler */		\
			NULL,		/* no get next for simples */	\
			SAGSimpleCheck,	/* check routine, if needed */	\
			SAGSimpleCommit,/* commit routine, if needed */	\
			NULL		/* abort is of no concern */	\
		    }							\
/* end of SAI_DEFINE_TABLE */

/* define access to an object organized as an array of pointers to values. */
#define	SAI_DEFINE_POINTER( Object_Name, Table_Location )			\
    extern ObjControl Object_Name = {					\
			OCBF_NONE,	/* no flags */			\
			Table_Location,	/* pointer to table */		\
			SAGPtrGet,	/* our get handler */		\
			NULL,		/* no get next for simples */	\
			SAGPtrCheck,	/* check routine, if needed */	\
			SAGPtrCommit,	/* commit routine, if needed */	\
			NULL		/* abort is of no concern */	\
		    }							\
/* end of SAI_DEFINE_POINTER */

/* define access to an object organized as a group of instances, in which
 * each individual instance is arranged as a simple array of SNMP values.
 */
#define	SAI_DEFINE_GROUP( Object_Name, Lookup_Routine )			\
    extern ObjControl Object_Name = {					\
		OCBF_NONE,		/* no flags */			\
		Lookup_Routine,		/* pointer to table */		\
		SAGSimpleLookupGet,	/* our get handler */		\
		SAGSimpleLookupGetNext,	/* get next */			\
		SAGSimpleLookupCheck,	/* check routine, if needed */	\
		SAGSimpleLookupCommit,	/* commit routine, if needed */	\
		NULL			/* still can't abort */		\
	    }								\
/* end of SAI_DEFINE_GROUP */



/* User Supplied Routines */
/* ---------------------- */


/*
$Name: OCBLookupInst - called to lookup an instance array instance $
*/

typedef ValueOrPtr *OCBLookupInst(
	struct ObjControl *ocb_pt,	/* control block */
	AttrValue	  *inst_list,	/* instance ID for this object */
	AttrValue	  *next_list	/* instance ID for next object */
);

/* 
Sample Call:	inst_array = (*(OCBLookupInst *)ocb_pt->ocb_context_pt)(
					ocb_pt,
					inst_list,
					next_list
				     );


$Description:
The generic processing support calls this routine to find a
particular instance out of a group of instances for an object
arranged as a group of simple attribute arrays.  A pointer to this
routine is placed in the ObjControl ocb_context_pt field by
SAI_DEFINE_GROUP.
$

$Return:
A pointer to the beginning of an attribute array which contains the
desired instance, or NULL if no instance fitting the description was
found.
$

$Bugs:
Cannot be used to create a new instances because the generic support
code won't handle the abort case.

The inst_list may be partial if this is a get-next.
$

$Method:
If next_list is NULL, just look for the instance identified by inst_list.
If next_list is not NULL, this is a get-next request, so find the next
instance after the (hypothetical) one described by inst_list, and return
the indentification of that instance in the next_list array.
$

*/



/* Externals */
/* --------- */

extern int SAGGetOneFromValue(
	ValueOrPtr	*value_pt,	/* value to use */
	AttrValue	*attr_pt	/* attribute to fill in */
);

extern int SAGGetOneFromArray(
	ValueOrPtr	*value_array,	/* array of possible values */
	AttrValue	*attr_pt	/* attribute to fill in */
);

extern int SAGGetOneFromPtrArray(
	ValueOrPtr	*ptrArray,	/* array of possible values */
	AttrValue	*attrPtr	/* attribute to fill in */
);

extern int SAGGetFromArray(		/* read attributes from an array */
	ValueOrPtr	*value_array,	/* array of values */
	AttrValue	*attr_list	/* list of attributes to get */
);

extern int SAGGetFromPtrArray(
	ValueOrPtr	*value_array,	/* array of pointers to values */
	AttrValue	*attr_list	/* list of attributes to get */
);

extern int SAGSetOneToValue(
	ValueOrPtr	*value_pt,	/* value to use */
	AttrValue	*attr_pt,	/* attribute containing info */
	int		really_set	/* TRUE if not just the check */
);

extern int SAGSetOneToArray(
	ValueOrPtr	*value_array,	/* array of possible values  */
	AttrValue	*attr_pt,	/* attribute containing info */
	int		really_set	/* TRUE if not just the check */
);

extern int SAGSetOneToPtrArray(
	ValueOrPtr	*ptrArray,	/* pointer array of possible values */
	AttrValue	*attrPtr,	/* attribute containing info */
	int		setFlag		/* TRUE if not just the check */
);

extern int SAGSetToArray(		/* set attributes to an array */
	ValueOrPtr	*value_array,	/* array of values */
	AttrValue	*attr_list,	/* list of attributes to get */
	int		really_set	/* TRUE if not just the check */
);

extern int SAGSetToPtrArray(
	ValueOrPtr	*value_array,	/* array of pointers to values */
	AttrValue	*attr_list,	/* list of attributes to get */
	int		setFlag		/* TRUE if not just the check */
);

extern int SAGSimpleGet(		/* OCBGet for simple array */
	struct ObjControl *ocb_pt,	/* control block */
	AttrValue	  *inst_list,	/* list of instance ID info */
	AttrValue	  *attr_list	/* list of attributes to get */
);

extern int SAGSimpleCheck(		/* OCBCheckSet for simple array */
	struct ObjControl *ocb_pt,	/* control block */
	AttrValue	  *inst_list,	/* list of instance ID info */
	AttrValue	  *attr_list,	/* list of attributes to get */
	void		  **handle_pp	/* place to return handle */
);

extern int SAGSimpleCommit(		/* OCBCommitSet for simple array */
	struct ObjControl *ocb_pt,	/* control block */
	AttrValue	  *inst_list,	/* list of instance ID info */
	AttrValue	  *attr_list,	/* list of attributes to get */
	void		  *handle_pt	/* place to return handle */
);

extern int SAGPtrGet(			/* OCBGet for pointer array */
	struct ObjControl *ocb_pt,	/* control block */
	AttrValue	  *inst_list,	/* list of instance ID info */
	AttrValue	  *attr_list	/* list of attributes to get */
);

extern int SAGPtrCheck(			/* OCBCheckSet for pointer array */
	struct ObjControl *ocb_pt,	/* control block */
	AttrValue	  *inst_list,	/* list of instance ID info */
	AttrValue	  *attr_list,	/* list of attributes to get */
	void		  **handle_pp	/* place to return handle */
);

extern int SAGPtrCommit(		/* OCBCommitSet for pointer array */
	struct ObjControl *ocb_pt,	/* control block */
	AttrValue	  *inst_list,	/* list of instance ID info */
	AttrValue	  *attr_list,	/* list of attributes to get */
	void		  *handle_pt	/* place to return handle */
);

extern int SAGSimpleLookupGetNext(	/* OCBGetNext for instance array */
	struct ObjControl *ocb_pt,	/* control block */
	AttrValue	  *inst_list,	/* instance ID for this object */
	AttrValue	  *next_list,	/* instance ID for next object */
	AttrValue	  *attr_list	/* list of attributes to get */
);

extern int SAGSimpleLookupGet(		/* OCBGet for instance array */
	struct ObjControl *ocb_pt,	/* control block */
	AttrValue	  *inst_list,	/* list of instance ID info */
	AttrValue	  *attr_list	/* list of attributes to get */
);

extern int SAGSimpleLookupSet(		/* basic set for instance array */
	struct ObjControl *ocb_pt,	/* control block */
	AttrValue	  *inst_list,	/* list of instance ID info */
	AttrValue	  *attr_list,	/* list of attributes to get */
	int		  really_set	/* TRUE: commit, FALSE: check */
);

extern int SAGSimpleLookupCheck(	/* OCBCheckSet for instance array */
	struct ObjControl *ocb_pt,	/* control block */
	AttrValue	  *inst_list,	/* list of instance ID info */
	AttrValue	  *attr_list,	/* list of attributes to get */
	void		  **handle_pp	/* place to return handle */
);

extern int SAGSimpleLookupCommit(	/* OCBCommitSet for instance array */
	struct ObjControl *ocb_pt,	/* control block */
	AttrValue	  *inst_list,	/* list of instance ID info */
	AttrValue	  *attr_list,	/* list of attributes to get */
	void		  *handle_pt	/* place to return handle */
);

#endif /* _SA_GENER_H_ */

/* end of generic.h */


