#ifndef _SYS_STAT_H_
#define _SYS_STAT_H_
/*============================================================================
=  NetWare C NLM Runtime Library source code
=
=  Unpublished Copyright (C) 1997 by Novell, Inc. All rights reserved.
=
=  No part of this file may be duplicated, revised, translated, localized or
=  modified in any manner or compiled, linked or uploaded or downloaded to or
=  from any computer system without the prior written consent of Novell, Inc.
=
=  sys/stat.h
==============================================================================
*/

/* POSIX file types */
#define S_IFMT       0170000  /* type of file (mask for following) */
#define S_IFIFO      0010000  /* first-in/first-out (not possible on NetWare) */
#define S_IFCHR      0020000  /* character-special file */
#define S_IFDIR      0040000  /* directory */
#define S_IFBLK      0060000  /* block file (not possible on NetWare) */
#define S_IFREG      0100000  /* regular */
#define S_IFLNK      0120000  /* symbolic link (not possible on NetWare) */
#define S_IFSOCK     0140000  /* Berkeley socket (not yet returned) */

#define S_ISFIFO(m)  (((m) & S_IFMT) == S_IFIFO)   /* should be FALSE */
#define S_ISCHR(m)   (((m) & S_IFMT) == S_IFCHR)
#define S_ISDIR(m)   (((m) & S_IFMT) == S_IFDIR)
#define S_ISBLK(m)   (((m) & S_IFMT) == S_IFBLK)   /* should be FALSE */
#define S_ISREG(m)   (((m) & S_IFMT) == S_IFREG)
#define S_ISLNK      (((m) & S_IFMT) == S_IFLNK)   /* should be FALSE */
#define S_ISSOCK     (((m) & S_IFMT) == S_IFSOCK)  /* currently FALSE */

/* POSIX file modes: owner (user) permission */
#define S_IRWXU      0000700
#define S_IRUSR      0000400
#define S_IWUSR      0000200
#define S_IXUSR      0000100

#define S_IREAD      S_IRUSR
#define S_IWRITE     S_IWUSR
#define S_IEXEC      S_IXUSR

/* POSIX file modes: group permission */
#define S_IRWXG      0000070
#define S_IRGRP      0000040
#define S_IWGRP      0000020
#define S_IXGRP      0000010

/* POSIX file modes: other permission */
#define S_IRWXO      0000007
#define S_IROTH      0000004
#define S_IWOTH      0000002
#define S_IXOTH      0000001

/* POSIX setuid, setgid, and sticky */
#define S_ISUID      0004000
#define S_ISGID      0002000
#define S_ISVTX      0001000


#ifndef _SIZE_T
# define _SIZE_T
typedef unsigned int size_t;
#endif

#ifndef _OFF_T
#  define _OFF_T
typedef long      off_t;
#endif

#ifndef _INO_T
# define _INO_T
typedef long      ino_t;
#endif

#ifndef _DEV_T
# define _DEV_T
typedef long      dev_t;
#endif

#ifndef _TIME_T
#  define _TIME_T
typedef unsigned long   time_t;
#endif

#ifndef _MODE_T
#  define _MODE_T
typedef unsigned long   mode_t;
#endif

struct stat 
{
   dev_t          st_dev;        /* volume number                             */
   ino_t          st_ino;        /* directory entry number of the st_name     */
   unsigned short st_mode;       /* emulated file mode                        */
   short          st_nlink;      /* count of hard links (always 1)            */
   unsigned long  st_uid;        /* object id of owner                        */
   short          st_gid;        /* group-id (always 0)                       */
   dev_t          st_rdev;       /* device type (always 0)                    */
   off_t          st_size;       /* total file size--files only               */
   time_t         st_atime;      /* last access date--files only              */
   time_t         st_mtime;      /* last modify date and time                 */
   time_t         st_ctime;      /* POSIX: last status change time...         */
                                 /* ...NetWare: creation date/time            */
   time_t         st_btime;      /* last archived date and time               */
   unsigned long  st_attr;       /* file attributes                           */
   unsigned long  st_archivedID; /* user/object ID of last archive            */
   unsigned long  st_updatedID;  /* user/object ID of last update             */
   unsigned short st_inheritedRightsMask;  /* inherited rights mask           */
   unsigned char  st_originatingNameSpace; /* namespace of creation           */
   unsigned char  st_name[255 + 1]; /* TARGET_NAMESPACE name                  */

   /*----------------- new fields starting in v4.11 ------------------------- */
   size_t         st_blksize;    /* block size for allocation--files only     */
   size_t         st_blocks;     /* count of blocks allocated to file         */
   unsigned int   st_flags;      /* user-defined flags                        */
   unsigned long  st_spare[4];   /* for future use                            */
};


#ifdef __cplusplus
extern "C" {
#endif

extern int chmod( const char *path, mode_t mode );
extern int fstat( int fildes, struct stat *buf );
extern int fstat_411( int fildes, struct stat *buf );
extern int mkdir( const char *path );
extern int stat( const char *path, struct stat *buf );
extern int stat_411( const char *path, struct stat *buf );
extern mode_t umask( mode_t cmask );

#ifdef __cplusplus
}
#endif


#define fstat(fd, buf)     fstat_411(fd, buf)
#define stat(path, buf)    stat_411(path, buf)

#endif
