/*
 *  Copyright (C) 1993-1995 Novell, Inc.  All Rights Reserved
 *  Copyright (C) 1995 Novell, Inc.  All Rights Reserved
 *
 *  THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND
 *  TREATIES. NO PART OF THIS WORK MAY BE USED, PRACTICED, PERFORMED, 
 *  COPIED, DISTRIBUTED, REVISED, MODIFIED, TRANSLATED, ABRIDGED, 
 *  CONDENSED, EXPANDED, COLLECTED, COMPILED, LINKED, RECAST, TRANSFORMED
 *  OR ADAPTED WITHOUT THE PRIOR WRITTEN CONSENT OF NOVELL, INC. ANY
 *  USE OR EXPLOITATION OF THIS WORK WITHOUT AUTHORIZATION COULD SUBJECT 
 *  THE PERPETRATOR TO CRIMINAL AND CIVIL LIABILITY.
 *  
 */

/*
 *  Copyright (c) 1985-1990 Apple Computer, Inc.  All Rights Reserved. 
 *
 *  The information contained herein is subject to change without
 *  notice and  should not be  construed as a commitment by Apple
 *  Computer, Inc. Apple Computer, Inc. assumes no responsibility
 *  for any errors that may appear.
 *
 *  Confidential and Proprietary to Apple Computer, Inc.
 */

/* @(#)atp.h: 1.0, 1.6; 1/4/90; Copyright 1985-90, Apple Computer, Inc. */

/* Definitions for ATP protocol and streams module, per 
 * AppleTalk Transaction Protocol documentation from
 * `Inside AppleTalk', July 14, 1986.
 */


#ifndef __ATP_H__
#define __ATP_H__

#include <appletlk.h>

#define ATP_HDR_SIZE            8       /* Size of the ATP header */
#define ATP_DATA_SIZE           578     /* Maximum size of the ATP data */

/* Timer values for XO release timers */
#define ATP_XO_30SEC            0
#define ATP_XO_1MIN             1
#define ATP_XO_2MIN             2
#define ATP_XO_4MIN             3
#define ATP_XO_8MIN             4

#define ATP_XO_DEF_REL_TIME     ATP_XO_30SEC

/*
 * ATP event types
 *
 * When a ATAtpGet returns, one of these functions is the event.
 * Sends will occur when an error condition, like ETIMEDOUT, occurs.
 */
#define ATP_EVENT_RCV_REQ       30
#define ATP_EVENT_RCV_RSP       31
#define ATP_EVENT_SND_REQ       32
#define ATP_EVENT_SND_RSP       33

/* ATP passing structure
 *
 * This structure will be used by both the library routines and the
 * streams functions. For STREAMS, the data will be held in the M_PROTO
 * portion of a putmsg/getmsg message. For across the library boundry, the
 * data will be pointed to by the structure.
 */
typedef struct ATAtpPass {
    u_short     event;
    int         ret;
    ATInet_t    at_addr;
    void        *TransID;
    u_char      xo;
    u_char      xo_relt;
    u_char      bitmap;
    u_char      packetize;
    ATRetry_t   retry;

    void        *data;
    u_short     data_len;

    u_long      userdata[8];
    u_short     packet_len[8];
} ATAtpPass_t;

#ifdef __cplusplus
extern "C" {
#endif

extern int ATAtpOpen(int *fd, ATSocket *socket);
extern int ATAtpClose(int fd);
extern int ATAtpGet(int fd, ATAtpPass_t *pass);
extern int ATAtpSendRsp(int fd, ATAtpPass_t *pass);
extern int ATAtpSendReq(int fd, ATAtpPass_t *pass);
extern int ATAtpCancelSendReq(int fd, void *TransID);
extern int ATAtpCancelRecvReq(int fd, void *TransID);

#ifdef __cplusplus
}
#endif

#endif /* __ATP__ */

