/*
//ͻ
// (c) 1993 Novell, Inc.	All rights reserved.                              
//																								  
// THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND		  
// TREATIES.																					  
//																								  
// NO PART OF THIS WORK MAY BE USED, PRACTICED, PERFORMED, COPIED,			  
// DISTRIBUTED, REVISED, MODIFIED, TRANSLATED, ABRIDGED, CONDENSED,         
// EXPANDED, COLLECTED, COMPILED, LINKED, RECAST, TRANSFORMED OR ADAPTED	  
// WITHOUT THE PRIOR WRITTEN CONSENT OF NOVELL, INC.                        
//																								  
// ANY USE OR EXPLOITATION OF THIS WORK WITHOUT AUTHORIZATION COULD SUBJECT 
// THE PERPETRATOR TO CRIMINAL AND CIVIL LIABILITY.								  
//͹
// Source module name: NWPSPDFX.H														  
//ͼ
*/
#ifdef __cplusplus
extern "C" {
#endif

/*++++++++++++++++++++++ General Database calls +++++++++++++++++++++*/
/*==================================================================
  Get the Version number stored in the database.  The database
  file/stream is automaticly opened and closed by this call.
  This call returns 0 on success, or non-zero error code on failure.
  ==================================================================*/
N_EXTERN_LIBRARY(NWCCODE)
NWPSPdfGetVersion(
  nuint		connType,							 /* Type of server/network           */
  nuint32	connID,							  /* NetWare Server Connection ID	  */
  pnuint32	pdfVersion);				 /* Returns db version				  */

/*==================================================================
  Set the Version number stored in the database.  The database
  file/stream is automaticly opened and closed by this call.
  This call returns 0 on success, or non-zero error code on failure.
  ==================================================================*/
N_EXTERN_LIBRARY(NWCCODE)
NWPSPdfSetVersion(
  nuint		connType,							 /* Type of server/network           */
  nuint32	connID,							  /* NetWare Server Connection ID	  */
  nuint32	pdfVersion);						 /* Returns db version				  */

/*=========================================================================
	Set the Context's DCK_NAME_CONTEXT to the Organizational Unit or
	Organization in the Directory where a database already exists, searching
	the Directory toward the [Root] from the given Printer or Print Queue.
  =========================================================================*/
N_EXTERN_LIBRARY(NWCCODE)
NWPSPdfLocateDBAndSetContext(
  nuint32	contextID,						  /* Directory Context Handle         */
  pnstr8    printerOrQueueObjectName ); /* search starting point			 */

/*++++++++++++++++++++++++++++ Form Calls +++++++++++++++++++++++++++++++++*/
/*==================================================================
  Add a form to the PrintDef database. Forms are independent
  of printers and are unique on each file server or context.
  This call returns 0 on success, or non-zero error code on failure.
  ==================================================================*/
N_EXTERN_LIBRARY(NWCCODE)
NWPSPdfAddForm(
  nuint		connType,							 /* Type of server/network           */
  nuint32	connID,							  /* NetWare Server Connection ID	  */
  pnstr8    formName,					 /* form name								 */
  nuint		formNumber,                    /* form number							 */
  nuint		formLength,                    /* form length							 */
  nuint		formWidth);                    /* form width                       */

/*===================================================================
  Delete a form from the PrintDef database.
  A zero is returned on succecss, or a non-zero error code on failure
  ===================================================================*/
N_EXTERN_LIBRARY(NWCCODE)
NWPSPdfDeleteForm(
  nuint		connType,					 /* Type of server/network           */
  nuint32	connID,						 /* NetWare Server Connection ID	    */
  pnstr8    formName);					 /* form name								 */

/*===================================================================
  Get each Form in turn; these calls obsoleted the ScanForm call
  ===================================================================*/
N_EXTERN_LIBRARY(NWCCODE)
NWPSPdfGetFirstForm(
  nuint		connType,					 /* Type of server/network           */
  nuint32	connID,						 /* NetWare Server Connection ID	    */
  NWPSListHandle NWPTR handle,		 /* Handle for GetNext and EndNext	 */
  pnstr8    formName,					 /* form name								 */
  pnuint    formNumber,              /* number or NULL						 */
  pnuint    formLength,              /* length space or NULL				 */
  pnuint    formWidth);              /* width space or NULL              */

N_EXTERN_LIBRARY(NWCCODE)
NWPSPdfGetNextForm(
  NWPSListHandle handle,				 /* Handle for GetNext and EndNext	 */
  pnstr8    formName,					 /* form name								 */
  pnuint    formNumber,              /* number or NULL						 */
  pnuint    formLength,              /* length space or NULL				 */
  pnuint    formWidth);              /* width space or NULL              */

/*======================================================
  Read the form information from the PrintDef database.
  If the form does not exist, an error code is returned.
  If it does exist, the form's name, number, length and
  width are set and a 0 is returned.
  ======================================================*/
N_EXTERN_LIBRARY(NWCCODE)
NWPSPdfReadForm(
  nuint		connType,					 /* Type of server/network           */
  nuint32	connID,						 /* NetWare Server Connection ID	    */
  pnstr8    formName,					 /* form name								 */
  pnuint    formNumber,              /* number or NULL						 */
  pnuint    formLength,              /* length space or NULL				 */
  pnuint    formWidth);              /* width space or NULL              */

/*===================================================================
  Update the form information. If you don't want to change the
  name, set newFormName to NULL. If you don't want to change a
  parameter set the value to -1.
  A zero is returned on succecss, or a non-zero error code on failure
  ===================================================================*/
N_EXTERN_LIBRARY(NWCCODE)
NWPSPdfUpdateForm(
  nuint		connType,		            /* Type of server/network        */
  nuint32	connID,							/* NetWare Server Connection ID  */
  pnstr8    oldFormName,				   /* old form name                 */
  pnstr8    newFormName,				   /* new form name or NULL			*/
  nuint		formNumber,                /* new form number or -1			*/
  nuint		formLength,                /* new form length or -1			*/
  nuint		formWidth);                /* new form width or -1				*/

/*+++++++++++++++++++++++++++ Device Calls ++++++++++++++++++++++++++++++++*/
/*===================================================================
  Add a device to the PrintDef database. The Device is created
  without any Functions or modes.
  A zero is returned on succecss, or a non-zero error code on failure
  ===================================================================*/
N_EXTERN_LIBRARY(NWCCODE)
NWPSPdfAddDevice(
  nuint		connType,						/* Type of server/network        */
  nuint32	connID,							/* NetWare Server Connection ID  */
  pnstr8    deviceName);			      /* device name							*/

/*===================================================================
  Delete a device from the PrintDef database.
  When a Device is deleted, all of the device's Functions and Modes
  are also deleted.
  A zero is returned on succecss, or a non-zero error code on failure
  ===================================================================*/
N_EXTERN_LIBRARY(NWCCODE)
NWPSPdfDeleteDevice(
  nuint		connType,						/* Type of server/network       */
  nuint32	connID,							/* NetWare Server Connection ID */
  pnstr8    deviceName);				   /* device to be removed			  */


/*===================================================================
  Get each Device in turn; these calls obsoleted the ScanDevice call
  ===================================================================*/
N_EXTERN_LIBRARY(NWCCODE)
NWPSPdfGetFirstDevice(
  nuint		connType,							 /* Type of server/network           */
  nuint32	connID,							  /* NetWare Server Connection ID	  */
  NWPSListHandle NWPTR handle,			/* Handle for GetNext and EndNext	*/
  pnstr8    deviceName);				 /* returned name found              */

N_EXTERN_LIBRARY(NWCCODE)
NWPSPdfGetNextDevice(
  NWPSListHandle handle,					/* Handle for GetNext and EndNext	*/
  pnstr8    deviceName);				 /* returned name found              */

/*===================================================================
  Find out how many functions and modes are defined for a device.
  A zero is returned on succecss, or a non-zero error code on failure
  ===================================================================*/
N_EXTERN_LIBRARY(NWCCODE)
NWPSPdfReadDevice(
  nuint		connType,							 /* Type of server/network           */
  nuint32	connID,							  /* NetWare Server Connection ID	  */
  pnstr8    deviceName,              /* name of device						 */
  pnuint    modeCount,					 /* number of modes defined			 */
  pnuint    funcCount);              /* number of func's defined			 */

/*===================================================================
  Change a Device name in the PrintDef database.
  A zero is returned on succecss, or a non-zero error code on failure
  ===================================================================*/
N_EXTERN_LIBRARY(NWCCODE)
NWPSPdfUpdateDevice(
  nuint		connType,							 /* Type of server/network           */
  nuint32	connID,							  /* NetWare Server Connection ID	  */
  pnstr8    oldDeviceName,           /* old device name						 */
  pnstr8    newDeviceName);			 /* new device name						 */

/*+++++++++++++++++++++++++ Mode Calls ++++++++++++++++++++++++++++++++++++*/
/*===================================================================
  Create a new Mode for a specific Device.
  A zero is returned on succecss, or a non-zero error code on failure
  ===================================================================*/
N_EXTERN_LIBRARY(NWCCODE)
NWPSPdfAddMode(
  nuint		connType,							 /* Type of server/network           */
  nuint32	connID,							  /* NetWare Server Connection ID	  */
  pnstr8    deviceName,              /* name of associated device        */
  pnstr8    modeName);					 /* new mode name                    */

/*===================================================================
  Delete a Mode from a Device.
  When the Mode is deleted, the Functions are left intact.
  A zero is returned on succecss, or a non-zero error code on failure
  ===================================================================*/
N_EXTERN_LIBRARY(NWCCODE)
NWPSPdfDeleteMode(
  nuint		connType,							 /* Type of server/network           */
  nuint32	connID,							  /* NetWare Server Connection ID	  */
  pnstr8    deviceName,              /* name of associated device        */
  pnstr8    modeName);					 /* name of mode to remove           */

/*===================================================================
  Get each Mode in turn; these calls obsoleted the ScanMode call
  ===================================================================*/
N_EXTERN_LIBRARY(NWCCODE)
NWPSPdfGetFirstMode(
  nuint		connType,							 /* Type of server/network           */
  nuint32	connID,							  /* NetWare Server Connection ID	  */
  pnstr8    deviceName,              /* name of associated device        */
  NWPSListHandle NWPTR handle,			/* Handle for GetNext and EndNext	*/
  pnstr8    modeName);					 /* returned mode name					 */

N_EXTERN_LIBRARY(NWCCODE)
NWPSPdfGetNextMode(
  NWPSListHandle handle,					/* Handle for GetFirst and EndNext	*/
  pnstr8    modeName);					 /* returned mode name					 */

/*===================================================================
  Find information on a defined Mode.
  If the Mode is defined, and funcCount is not NULL, the number
  of functions in the mode is returned in funcCount.
  A zero is returned on succecss, or a non-zero error code on failure
  ===================================================================*/
N_EXTERN_LIBRARY(NWCCODE)
NWPSPdfReadMode(
  nuint		connType,							 /* Type of server/network           */
  nuint32	connID,							  /* NetWare Server Connection ID	  */
  pnstr8    deviceName,              /* name of associated device        */
  pnstr8    reqModeName,				 /* request mode name					 */
  pnuint    funcCount);              /* function count or NULL           */

/*===================================================================
  Change the name of a Mode.
  A zero is returned on succecss, or a non-zero error code on failure
  ===================================================================*/
N_EXTERN_LIBRARY(NWCCODE)
NWPSPdfUpdateMode(
  nuint		connType,							 /* Type of server/network           */
  nuint32	connID,							  /* NetWare Server Connection ID	  */
  pnstr8    deviceName,              /* name of associated device        */
  pnstr8    oldModeName,				 /* old mode name                    */
  pnstr8    newModeName);				 /* new mode name or NULL				 */


/*++++++++++++++++++ Mode-Function Grouping Calls +++++++++++++++++++++++++*/
/*====================================================================
  Add a previously defined function to a previously defined mode list.
  A zero is returned on succecss, or a non-zero error code on failure
  ====================================================================*/
N_EXTERN_LIBRARY(NWCCODE)
NWPSPdfAddModeFunction(
  nuint		connType,							 /* Type of server/network           */
  nuint32	connID,							  /* NetWare Server Connection ID	  */
  pnstr8    deviceName,              /* name of associated device        */
  pnstr8    modeName,					 /* name of associated mode			 */
  pnstr8    funcName,					 /* func to add to group				 */
  nuint		location);							 /* where to insert function or -1	 */

/*==================================================================
  Delete a function from a defined mode list.
  Neither the Mode or Function is deleted from the Device lists.
  A zero is returned on success, or a non-zero error code on failure
  ==================================================================*/
N_EXTERN_LIBRARY(NWCCODE)
NWPSPdfDeleteModeFunction(
  nuint		connType,							 /* Type of server/network           */
  nuint32	connID,							  /* NetWare Server Connection ID	  */
  nuint32	sequence,							 /* number of the function to remove 
														from group								*/
  pnstr8    deviceName,              /* name of associated device        */
  pnstr8    modeName);					 /* name of associated mode			 */

/*===================================================================
  Get each Function in turn; these calls obsoleted the ScanModeFunction call
  ===================================================================*/
N_EXTERN_LIBRARY(NWCCODE)
NWPSPdfGetFirstModeFunction(
  nuint		connType,							 /* Type of server/network           */
  nuint32	connID,							  /* NetWare Server Connection ID	  */
  pnstr8    deviceName,              /* name of associated device        */
  pnstr8    modeName,					 /* name of associated mode			 */
  NWPSListHandle NWPTR handle,			/* Handle for GetNext and EndNext	*/
  pnstr8    funcName);					 /* returned function name           */

N_EXTERN_LIBRARY(NWCCODE)
NWPSPdfGetNextModeFunction(
  NWPSListHandle handle,					/* Handle for GetFirst and EndNext	*/
  pnstr8    funcName);					 /* returned function name           */

/*==================================================================
  Find the Modes which use this Function.
  To find all of the Modes associated with a Function, set
  sequence to -1 on the first call and it will be reset by
  the call if a Mode is found.
  A zero is returned on success, or a non-zero error code on failure
  ==================================================================*/
N_EXTERN_LIBRARY(NWCCODE)
NWPSPdfScanFunctionMode(
  nuint		connType,
  nuint32	connID,
  pnuint32	funcSequence,
  pnuint32	modeSequence,
  pnstr8    deviName,
  pnstr8    modeName,
  pnstr8    funcName);

/*=================================================================
  Read the actual function values associated with a mode.
  On the first call, funcOffset should be set to 0, and funcSize
  should be set to the size of the buffer pointed to by funcBuffer.
  On return funcSize will contain the actual number of bytes copied
  to the funcBuffer and the return value will be 0.
  A zero is returned if there is any data after the specified
  funcOffset (start point).  A failure code is retuned if
  there is an error or if funcOffset is greater than the
  available number of bytes.
  =================================================================*/
N_EXTERN_LIBRARY(NWCCODE)
NWPSPdfReadModeFunction(
  nuint		connType,							 /* Type of server/network           */
  nuint32	connID,							  /* NetWare Server Connection ID	  */
  pnstr8    deviceName,              /* name of associated device        */
  pnstr8    modeName,					 /* name of associated mode			 */
  /*-------- no function name because this is a mode group ----------------*/
  nuint		funcOffset,                    /* number of bytes to skip			 */
  pnuint    funcSize,					 /* req:buff size; ret:# read        */
  pnuint8	funcString);				 /* buffer for read block			  */

/*+++++++++++++++++++++++++++ Function Calls ++++++++++++++++++++++++++++++*/
/*===================================================================
  Add a Function to a Device.
  funcSize should be set to the number of bytes in funcString.
  A zero is returned on succecss, or a non-zero error code on failure
  ===================================================================*/
N_EXTERN_LIBRARY(NWCCODE)
NWPSPdfAddFunction(
  nuint		connType,							 /* Type of server/network           */
  nuint32	connID,							  /* NetWare Server Connection ID	  */
  pnstr8    deviceName,              /* name of associated device        */
  pnstr8    funcName,					 /* name of func. to add				 */
  nuint		funcSize,							 /* count of bytes in function		 */
  pnuint8	funcString);				 /* list of bytes in function         */

/*===================================================================
  Delete a function from the Device in the PrintDef database.
  If the Function is refered to in any Mode, the Function
  will be automaticly deleted from the Mode as well.
  A zero is returned on succecss, or a non-zero error code on failure
  ===================================================================*/
N_EXTERN_LIBRARY(NWCCODE)
NWPSPdfDeleteFunction(
  nuint		connType,							 /* Type of server/network           */
  nuint32	connID,							  /* NetWare Server Connection ID	  */
  pnstr8    deviceName,              /* name of associated device        */
  pnstr8    funcName);					 /* name of func. to delete			 */

/*===================================================================
  Get each Function in turn; these calls obsoleted the ScanFunction call
  ===================================================================*/
N_EXTERN_LIBRARY(NWCCODE)
NWPSPdfGetFirstFunction(
  nuint		connType,							 /* Type of server/network           */
  nuint32	connID,							  /* NetWare Server Connection ID	  */
  pnstr8    deviceName,              /* name of associated device        */
  NWPSListHandle NWPTR handle,			/* Handle for GetNext and EndNext	*/
  pnstr8    funcName,					 /* returned function name           */
  NWPSPdfFuncData NWPTR funcData);		/* returned function data struct    */

N_EXTERN_LIBRARY(NWCCODE)
NWPSPdfGetNextFunction(
  NWPSListHandle handle,					/* Handle for GetFirst and EndNext	*/
  pnstr8    funcName,					 /* returned function name           */
  NWPSPdfFuncData NWPTR funcData);		/* returned function data struct    */

/*===================================================================
  Read the byte string associated with a function.
  funcSize should be set equal to the size of the buffer funcString.
  funcSize will be set to the actual size of the byte string if the
  call is successful.
  A zero is returned on succecss, or a non-zero error code on failure
  ===================================================================*/
N_EXTERN_LIBRARY(NWCCODE)
NWPSPdfReadFunction(
  nuint		connType,							 /* Type of server/network           */
  nuint32	connID,							  /* NetWare Server Connection ID	  */
  pnstr8    deviceName,              /* name of associated device        */
  pnstr8    funcName,					 /* name of the function				 */
  nuint		funcOffset,                    /* bytes to skip past					 */
  pnuint    funcSize,					 /* req: buff size; ret:# read		 */
  pnuint8	funcString);				 /* byte list or NULL					 */

/*===================================================================
  Change the function string assigned to a function name.
  To change the function's name, set newFuncName to a new name.
  To leave the Function name the same, set newFuncName to NULL.
  To change the function string, set funcSize to the number of
  bytes in the funcString and set funcString to point to the
  new string. To leave the string the same, set funcSize to -1.
  A zero is returned on succecss, or a non-zero error code on failure
  ===================================================================*/
N_EXTERN_LIBRARY(NWCCODE)
NWPSPdfUpdateFunction(
  nuint		connType,							 /* Type of server/network           */
  nuint32	connID,							  /* NetWare Server Connection ID	  */
  pnstr8    deviceName,              /* name of associated device        */
  pnstr8    oldFuncName,				 /* current function name				 */
  pnstr8    newFuncName,				 /* new name or NULL                 */
  nuint		funcSize,							 /* number of bytes or -1				 */
  pnuint8	funcString);				 /* byte list or NULL					 */

/*++++++++++++++++++++ Import and Export Functions ++++++++++++++++++++++++*/
/*-----------------------------------------------------
  The path name for a PDF file should have
  one of the following formats:
	 "\\<file server>\<volume>\<path>\<file name>[.PDF]"
	 "<volume>:<path>\<file name>[.PDF]"
	 NULL to create the file ".\<device name>[.PDF]"
  Every PDF import/export file also has a date code
  associated with it.  The date is always set when the
  file is created.
  -----------------------------------------------------*/
/*===========================================
  Import a device from a file to the database
  ===========================================*/
N_EXTERN_LIBRARY(NWCCODE)
NWPSPdfImportDevice(
  nuint		connType,							 /* Type of server/network           */
  nuint32	connID,							  /* NetWare Server Connection ID	  */
  pnstr8    fileName,					 /* name of .PDF file					 */
  pnstr8    deviceName);				 /* new device name or NULL			 */

N_EXTERN_LIBRARY(NWCCODE)
NWPSPdfManagedImportDevice(
  nuint		connType,							 /* Type of server/network           */
  nuint32	connID,							  /* NetWare Server Connection ID	  */
  pnstr8    fileName,					 /* name of .PDF file					 */
  pnstr8    deviceName,              /* new device name or NULL			 */
  NWPSImportManager importManagerFunc, /* pointer to the Manager Function	*/
  nptr		importManagerParm1);     /* caller's Manager context variable*/


/*===============================================
  Export a device to a Pdf file from the database
  ===============================================*/
N_EXTERN_LIBRARY(NWCCODE)
NWPSPdfExportDevice(
  nuint		connType,					 /* Type of server/network           */
  nuint32	connID,						 /* NetWare Server Connection ID     */
  pnstr8    fileName,					 /* name of .PDF file					 */
  pnstr8    deviceName);				 /* name of device to export			 */

/*=========================================================
  Delete the Directory Services "Printer Control" Attribute
  at the given context.
  =========================================================*/
N_EXTERN_LIBRARY(NWCCODE)
NWPSPdfDeleteDatabase(
  nuint		connType,					 /* Type of server/network           */
  nuint32	connID);                 /* NetWare Server Connection ID     */

#ifdef __cplusplus
}
#endif
