/*
//ͻ
// (c) 1993 Novell, Inc.	 All rights reserved.										
//																								  
// THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND		  
// TREATIES.																						
//																								  
// NO PART OF THIS WORK MAY BE USED, PRACTICED, PERFORMED, COPIED,				
// DISTRIBUTED, REVISED, MODIFIED, TRANSLATED, ABRIDGED, CONDENSED,         
// EXPANDED, COLLECTED, COMPILED, LINKED, RECAST, TRANSFORMED OR ADAPTED		
// WITHOUT THE PRIOR WRITTEN CONSENT OF NOVELL, INC.                        
//																								  
// ANY USE OR EXPLOITATION OF THIS WORK WITHOUT AUTHORIZATION COULD SUBJECT 
// THE PERPETRATOR TO CRIMINAL AND CIVIL LIABILITY.									
//͹
// Source module name: NWPS_COM.H															
//ͼ
*/
#ifndef NWPS_DEF_INC
#include <nwps_def.h>
#endif


#ifndef NWPS_COM_INC
#define NWPS_COM_INC
/*********************************************************************/
/*********** COM - Pserver Transport Interface ***********************/
/*********************************************************************/

/*========= Client privilege levels from the Print Server ================= */
#define NWPS_LIMITED                   0	  /* Limited access only			 */
#define NWPS_USER                      1	  /* User access						 */
#define NWPS_OPERATOR						2	  /* Operator access              */

/*========================== Job Outcomes ================================= */
#define NWPS_PLACE_ON_HOLD             0	  /* Place job on hold				 */
#define NWPS_RETURN_TO_QUEUE				1	  /* Return job to queue			 */
#define NWPS_THROW_AWAY                2	  /* Throw job away					 */

/*=================== Network Printer Status codes ======================== */
#define NWPS_PRINTER_RUNNING				0	  /* Printer is running           */
#define NWPS_PRINTER_OFFLINE				1	  /* Printer is offline           */
#define NWPS_PRINTER_PAPER_OUT			2	  /* Printer is out of paper		 */
#define NWPS_CONNECTION_LOST				4	  /* NPrinter connection was lost */

/*===== Network Printer and Extended Network Printer Info flags ============*/
#define NWPS_PRINTER_SHARED				0	 /* NPrinter is shared with net	 */
#define NWPS_PRINTER_PRIVATE				1	 /* NPrinter is private to ws     */

/*=== Print Server info structure returned by NWPSComGetPrintServerInfo ====*/
typedef struct
{
  nuint8 status,								 /* Print server status               */
			numPrinters,						 /* Number of attached printers		 */
			numModes,							 /* Number of queue service modes	 */
			majorVersion,						 /* Print server protocol, major vers*/
			minorVersion,						 /* Print server protocol, minor vers*/
			revision,							 /* Print server protocol, revision   */
			serialNumber[4],					 /* Serial number in BCD				 */
			serverType,                     /* Print Server Type
														0 - Unknown            
														1 - Dedicate print server for DOS
														2 - NetWare Loadable Module	 
														3 - VAP, in file server  
														4 - VAP, in Bridge	
														5 - Unix print server				*/
			nameServiceMode;					 /* NWPS_BINDERY_SERVICE or
														NWPS_DIRECTORY_SERVICE				*/
  nuint8 futureUse[8];						 /* Reserved for future use			 */
} NWPS_PSInfo;

/*==============================================
  Network Printer information structure returned
  by NWPSComRequestNetworkPrinter()
  ==============================================*/
typedef struct
{
  nuint16	printerType,						 /* Type of Network printer			 */
				useInterrupts,                  /* Should we use interrupts?			*/
				irqNumber,							 /* IRQ number for printer            */
				numBlocks,							 /* Number of blocks in buffer		 */
				useXonXoff,                     /* Use Xon/Xoff?		 (Serial only) */
				baudRate,							 /* Baud rate			  (Serial only) */
				dataBits,							 /* Number of bits	  (Serial only) */
				stopBits,							 /* Number of bits	  (Serial only) */
				parity,								 /* Parity type		  (Serial only) */
				socket;								 /* Socket number for Network printer*/
} NWPS_NInfo;

/*===============================================
  Network Printer status structure used to report
  the current status to the print server.
  (Not used in any API calls)
  ===============================================*/
typedef struct
{
  nuint8 printerNumber,                  /* Network printer number				*/
			needBlocks,                     /* Number of blocks needed to fill 
														buffers									 */
			finishedBlocks,					 /* Number of blocks printed			 */
			status,								 /* Printer online, offline, or 
														out-of-paper							*/
			inSideBand;                     /* True-NPrinter is in sideband mode*/
} NWPS_NStatus;

/*======================================================
  Data types sent by the PServer to a Network printer.
  The first byte of every print server to printer packet
  will have one of these codes in it.
  (Not used in any API calls)
  ======================================================*/
#define NWPS_DST_DATA						(nuint8) 0
#define NWPS_DST_FLUSH						(nuint8) 1
#define NWPS_DST_PAUSE						(nuint8) 2
#define NWPS_DST_START						(nuint8) 3
#define NWPS_DST_SIDEBAND					(nuint8) 4
#define NWPS_DST_NEW_JOB					(nuint8) 5
#define NWPS_DST_RELEASE					(nuint8) 6
#define NWPS_DST_RECLAIM					(nuint8) 7
#define NWPS_DST_EOJ                   (nuint8) 8



/*
// The following conditional includes pick up the varying definitions
// of NWPSComAttachToPrintServer and NWPSComLoginToPrintServer.  Since
// the majority of the NWPSComxxxxx calls are unaffected by the
// distinction between connections and their types the prototype were
// kept in this file.  This is an exception compared to the other
// major groups of supported calls.
*/
#if defined(NWPS_CONN_TYPE)
	#if NWPS_CONN_TYPE == NWPS_BINDERY_SERVICE
		#include <nwpscom0.h>
	#elif NWPS_CONN_TYPE == NWPS_DIRECTORY_SERVICE
		#include <nwpscom1.h>
	#elif NWPS_CONN_TYPE == NWPS_BINDERY_SERVICE_PRE_40
		#include <nwpscom0.h>
	#elif NWPS_CONN_TYPE == NWPS_BINDERY_SERVICE_ALL
		#include <nwpscom0.h>
	#endif
#else
	#include <nwpscomx.h>
#endif

#ifdef __cplusplus
	extern "C" {
#endif

/*====================== Print Server Communications ======================*/

N_EXTERN_LIBRARY(NWCCODE)
NWPSComAbortPrintJob(
	NWPS_SPXID_HANDLE spxID,					/* SPX Connection number				*/
	nuint             printerID,				/* Printer number                   */
	nuint             jobOutcome);			/* Job outcome                      */

N_EXTERN_LIBRARY(NWCCODE)
NWPSComAddNotifyObject(
	NWPS_SPXID_HANDLE spxID,					/* SPX Connection number				*/
	nuint             printerID,				/* Printer number                   */
	pnstr8				nServerName,			/* NetWare server name					 */
	pnstr8				objectName,          /* Object name                      */
	nuint             objectType,          /* Object type                      */
	nuint             notifyDelay,			/* First notify delay					*/
	nuint             notifyInterval);		/* Notify interval						*/

N_EXTERN_LIBRARY(NWCCODE)
NWPSComAddQueueToPrinter(
	NWPS_SPXID_HANDLE spxID,					/* SPX Connection number				*/
	nuint             printerID,				/* Printer number                   */
	pnstr8				nServerName,			/* NetWare server name					 */
	pnstr8				queueName,				/* Queue name								 */
	nuint             priority);				/* Priority                         */

N_EXTERN_LIBRARY(NWCCODE)
NWPSComAttachPServerToNServer(
	NWPS_SPXID_HANDLE spxID,					/* SPX Connection number				*/
	pnstr8				nServerName,			/* NetWare server name					 */
	pnstr8				password);				/* Password                         */

N_EXTERN_LIBRARY(NWCCODE)
NWPSComCancelDownRequest(
	NWPS_SPXID_HANDLE spxID);					/* SPX Connection number				*/

N_EXTERN_LIBRARY(NWCCODE)
NWPSComChangeNotifyInterval(
	NWPS_SPXID_HANDLE spxID,					/* SPX Connection number				*/
	nuint             printerID,				/* Printer number                   */
	pnstr8				nServerName,			/* NetWare server name					 */
	pnstr8				objectName,          /* Object name                      */
	nuint             objectType,          /* Object type                      */
	nuint             notifyDelay,			/* First notify delay					*/
	nuint             notifyInterval);		/* Notify interval						*/

N_EXTERN_LIBRARY(NWCCODE)
NWPSComChangeQueuePriority(
	NWPS_SPXID_HANDLE spxID,					/* SPX Connection number				*/
	nuint             printerID,				/* Printer number                   */
	pnstr8				nServerName,			/* NetWare server name					 */
	pnstr8				queueName,				/* Queue name								 */
	nuint             priority);				/* New priority							*/

N_EXTERN_LIBRARY(NWCCODE)
NWPSComChangeServiceMode(
	NWPS_SPXID_HANDLE spxID,					/* SPX Connection number				*/
	nuint             printerName,			/* Printer number                   */
	nuint             serviceMode);			/* New service mode						 */

N_EXTERN_LIBRARY(NWCCODE)
NWPSComDeleteNotifyObject(
	NWPS_SPXID_HANDLE spxID,					/* SPX Connection number				*/
	nuint             printerID,				/* Printer number                   */
	pnstr8				nServerName,			/* NetWare server name					 */
	pnstr8				objectName,          /* Object name                      */
	nuint             objectType);			/* Object type                      */

N_EXTERN_LIBRARY(NWCCODE)
NWPSComDeleteQueueFromPrinter(
	NWPS_SPXID_HANDLE spxID,					/* SPX Connection number				*/
	nuint             printerID,				/* Printer number                   */
	pnstr8				nServerName,			/* NetWare server name					 */
	pnstr8				queueName,				/* Queue name								 */
	nuint             detach,					/* Detach immediately?					 */
	nuint             jobOutcome);			/* Outcome of current job				 */

N_EXTERN_LIBRARY(NWCCODE)
NWPSComDetachFromPrintServer(
	NWPS_SPXID_HANDLE spxID);					/* SPX Connection number				*/

N_EXTERN_LIBRARY(NWCCODE)
NWPSComDetachPServerFromNServer(
	NWPS_SPXID_HANDLE spxID,					/* SPX Connection number				*/
	pnstr8				nServerName,			/* NetWare server name					 */
	nuint             detach,					/* Detach immediately?					 */
	nuint             jobOutcome);			/* Outcome of current jobs          */

N_EXTERN_LIBRARY(NWCCODE)
NWPSComDownPrintServer(
	NWPS_SPXID_HANDLE spxID,					/* SPX Connection number				*/
	nuint             immediate,				/* Go down immediately?             */
	nuint             jobOutcome);			/* Outcome of current jobs          */

N_EXTERN_LIBRARY(NWCCODE)
NWPSComEjectForm(
	NWPS_SPXID_HANDLE spxID,					/* SPX Connection number				*/
	nuint             printerID);          /* Printer number                   */

N_EXTERN_LIBRARY(NWCCODE)
NWPSComGetAttachedNServers(
	NWPS_SPXID_HANDLE spxID,					/* SPX Connection number				*/
	pnuint				sequence,				/* Sequence number. 0 first time    */
	pnstr8				nServerName);			/* NetWare server name					 */

N_EXTERN_LIBRARY(NWCCODE)
NWPSComGetNotifyObject(
	NWPS_SPXID_HANDLE spxID,					/* SPX Connection number				*/
	nuint             printerID,				/* Printer number                   */
	pnuint				sequence,				/* Sequence number.	 0 first time	 */
	pnstr8				nServerName,			/* NetWare server name					 */
	pnstr8				objectName,          /* Object name                      */
	pnuint				objectType,          /* Object type                      */
	pnuint				notifyDelay,			/* First notify delay					*/
	pnuint				notifyInterval);		/* Notify interval						*/

N_EXTERN_LIBRARY(NWCCODE)
NWPSComGetPrintersServicingQ(
	NWPS_SPXID_HANDLE spxID,					/* SPX Connection number				*/
	pnstr8				nServerName,			/* NetWare server name					 */
	pnstr8				queueName,				/* Queue name								 */
	nuint             maxPrinters,			/* Maximum # of returned printers	*/
	pnuint				actualPrinters,		/* Actual # of returned printers    */
	pnuint				printerArray);       /* Array for returned printer #s    */

N_EXTERN_LIBRARY(NWCCODE)
NWPSComGetPrinterStatus(
	NWPS_SPXID_HANDLE spxID,					/* SPX Connection number				*/
	nuint             printerID,				/* Printer number                   */
	pnuint				status,					/* Printer status                   */  
	pnuint				troubleCode,			/* On line/Off line/Out of paper    */  
	pnuint				active,					/* Printer has an active job			 */  
	pnuint				serviceMode,			/* Queue service mode					*/
	pnuint				formNumber,          /* Mounted form number					 */
	pnstr8				formName,				/* Mounted form name                */
	pnstr8				printerName);			/* Printer name							*/

N_EXTERN_LIBRARY(NWCCODE)
NWPSComGetExtPrinterStatus(
	NWPS_SPXID_HANDLE spxID,					/* SPX Connection number				*/
	nuint             printerID,				/* Printer number                   */
	pnstr8				primaryStatus,       /* Primary status (60max)				 */
	pnuint				primaryLevel,			/* Primary error level					 */
	pnstr8				secondaryStatus,		/* Secondary status (60max)			*/
	pnuint				secondaryLevel,		/* Secondary error level				*/
	pnuint				activeJobCount,		/* Number of active jobs				*/
	pnuint				serviceMode,			/* Queue service mode					*/
	pnuint				formsMounted,			/* Number of forms mounted          */
	pnuint				formList);				/* Array of mounted forms (20max)	*/

N_EXTERN_LIBRARY(NWCCODE)
NWPSComGetPrintJobStatus(
	NWPS_SPXID_HANDLE spxID,					/* SPX Connection number				*/
	nuint             printerID,				/* Printer number                   */
	pnstr8				nServerName,			/* NetWare server name					 */  
	pnstr8				queueName,				/* Queue name								 */  
	pnuint32          jobID,					/* Job number								 */
	pnstr8				jobDescription,		/* Description of job					*/
	pnuint				copies,					/* Number of copies to be printed	*/
	pnuint32          printJobSize,			/* Size of print job                */
	pnuint				copiesDone,          /* Copies finished						*/
	pnuint32          bytesDone,				/* Bytes into current copy          */
	pnuint				formNumber,          /* Form number for job					 */
	pnuint				textFlag);				/* Is job text?							*/

N_EXTERN_LIBRARY(NWCCODE)
NWPSComGetPrintServerInfo(
	NWPS_SPXID_HANDLE spxID,					/* SPX Connection number				*/
	NWPS_PSInfo NWPTR psInfo,					/* Server info structure				*/
	nuint             size);					/* Size of information requested    */

N_EXTERN_LIBRARY(NWCCODE)
NWPSComGetQueuesServiced(
	NWPS_SPXID_HANDLE spxID,					/* SPX Connection number				*/
	nuint             printerID,				/* Printer number                   */
	pnuint				sequence,				/* Sequence number.	 0 first time	 */
	pnstr8				nServerName,			/* NetWare server name					 */
	pnstr8				queueName,				/* Queue name								 */
	pnuint				priority);				/* Priority                         */

N_EXTERN_LIBRARY(NWCCODE)
NWPSComMarkTopOfForm(
	NWPS_SPXID_HANDLE spxID,					/* SPX Connection number				*/
	nuint             printerID,				/* Printer number                   */
	char					mark);					/* Character to mark form with		*/

N_EXTERN_LIBRARY(NWCCODE)
NWPSComPrintServerRequest(
	NWPS_SPXID_HANDLE spxID,					/* SPX Connection number				*/
	nptr					reqBuffer,				/* address of request packet buffer */
	int					reqSize,             /* length of request packet			*/
	nptr					repBuffer,				/* address of reply packet buffer	*/
	int					repSize);				/* length of expected reply			*/

N_EXTERN_LIBRARY(NWCCODE)
NWPSComRewindPrintJob(
	NWPS_SPXID_HANDLE spxID,					/* SPX Connection number				*/
	nuint             printerID,				/* Printer number                   */
	nuint             byPage,					/* Rewind by page?						*/
	nuint             relative,				/* Rewind relative to cur. position?*/
	nuint             copy,                /* Copy to rewind to (if absolute)	 */
	nuint32				offset);             /* Offset									*/

N_EXTERN_LIBRARY(NWCCODE)
NWPSComSetMountedForm(
	NWPS_SPXID_HANDLE spxID,					/* SPX Connection number				*/
	nuint             printerID,				/* Printer number                   */
	nuint             formNumber);			/* Form number                      */

N_EXTERN_LIBRARY(NWCCODE)
NWPSComStartPrinter(
	NWPS_SPXID_HANDLE spxID,					/* SPX Connection number				*/
	nuint             printerID);          /* Printer number                   */

N_EXTERN_LIBRARY(NWCCODE)
NWPSComStopPrinter(
	NWPS_SPXID_HANDLE spxID,					/* SPX Connection number				*/
	nuint             printerID,				/* Printer number                   */
	nuint             jobOutcome);			/* Job outcome                      */

/*================== Network Printer Calls ===============================*/
N_EXTERN_LIBRARY(NWCCODE)
NWPSComGetNextRemotePrinter(
	NWPS_SPXID_HANDLE spxID,					/* SPX Connection number				*/
	pnuint				printerID,				/* Printer number                   */
	pnuint				printerType,			/* Printer type							*/
	pnstr8				printerName);			/* Name of printer						*/

N_EXTERN_LIBRARY(NWCCODE)
NWPSComRequestRemotePrinter(
	NWPS_SPXID_HANDLE spxID,					/* SPX Connection number				*/
	nuint             printerID,				/* Printer number                   */
	NWPS_NInfo NWPTR	info);					/* Network printer info structure	*/

N_EXTERN_LIBRARY(NWCCODE)
NWPSComSetRemoteMode(
	NWPS_SPXID_HANDLE spxID,					/* SPX Connection number				*/
	nuint             printerID,				/* Printer number                   */
	nuint             newMode);				/* New mode                         */

#ifdef __cplusplus
}
#endif

#endif  /* NWPS_COM_INC */

