#ifndef __sys_mode_h__
#define __sys_mode_h__
/*============================================================================
=  Novell Software Development Kit
=
=  Copyright (C) 1999-2002 Novell, Inc. All Rights Reserved.
=
=  This work is subject to U.S. and international copyright laws and treaties.
=  Use and redistribution of this work is subject  to  the  license  agreement
=  accompanying  the  software  development kit (SDK) that contains this work.
=  However, no part of this work may be revised and/or  modified  without  the
=  prior  written consent of Novell, Inc. Any use or exploitation of this work
=  without authorization could subject the perpetrator to criminal  and  civil
=  liability.
=
=  Source(s): UNIX-inspired header
=
=  sys/mode.h
==============================================================================
*/

/* POSIX file types... */
#define S_IFMT       0xF000   /* type of file (mask for following)      */
#define S_IFIFO      0x1000   /* first-in/first-out (FIFO/pipe)         */
#define S_IFCHR      0x2000   /* character-special file (unused)        */
#define S_IFDIR      0x4000   /* directory                              */
#define S_IFBLK      0x6000   /* blocking device (unused)               */
#define S_IFREG      0x8000   /* regular                                */
#define S_IFLNK      0xA000   /* symbolic link (unused)                 */
#define S_IFSOCK     0xC000   /* Berkeley socket                        */

#define S_ISFIFO(m)  (((m) & S_IFMT) == S_IFIFO)  /* e.g.: FIFO/pipe    */
#define S_ISCHR(m)   (((m) & S_IFMT) == S_IFCHR)  /* e.g.: console      */
#define S_ISDIR(m)   (((m) & S_IFMT) == S_IFDIR)  /* e.g.: subdirectory */
#define S_ISBLK(m)   (((m) & S_IFMT) == S_IFBLK)
#define S_ISREG(m)   (((m) & S_IFMT) == S_IFREG)
#define S_ISLNK(m)   (((m) & S_IFMT) == S_IFLNK)  /* should be FALSE    */
#define S_ISSOCK(m)  (((m) & S_IFMT) == S_IFSOCK) /* e.g.: socket       */

/* POSIX file modes: owner (user) permission... */
#define S_IRWXU      0x01C0
#define S_IRUSR      0x0100
#define S_IWUSR      0x0080
#define S_IXUSR      0x0040

#define S_IREAD      S_IRUSR
#define S_IWRITE     S_IWUSR
#define S_IEXEC      S_IXUSR

/* POSIX file modes: group permission... */
#define S_IRWXG      0x0038
#define S_IRGRP      0x0020
#define S_IWGRP      0x0010
#define S_IXGRP      0x0008

/* POSIX file modes: other permission... */
#define S_IRWXO      0x0007
#define S_IROTH      0x0004
#define S_IWOTH      0x0002
#define S_IXOTH      0x0001

/* Novell-defined additional directory modes for mkdir()... */
#define S_DSYSTEM    0x00100000  /* system directory       */
#define S_DHIDE      0x00200000  /* hidden directory       */
#define S_DDEL_INH   0x00400000  /* delete-inhibit         */
#define S_DREN_INH   0x00800000  /* rename-inhibit         */
#define S_DPURGE_IMM 0x01000000  /* purge-immediate        */
#define S_DCOMP_IMM  0x02000000  /* compress-immediate     */
#define S_DCOMP_NO   0x04000000  /* no compression         */
#define S_DALL       0x01FB      /* equivalent to mode = 0 */

/* POSIX setuid(), setgid(), and sticky... */
#define S_ISUID      0004000
#define S_ISGID      0002000
#define S_ISVTX      0001000


#endif
