/*
  +************************************************************************+
  | Copyright (c) 1995 Novell, Inc.  All rights reserved.                  |
  |                                                                        |
  | THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND      |
  | TREATIES.                                                              |
  |                                                                        |
  | NO PART OF THIS WORK MAY BE USED, PRACTICED, PERFORMED, COPIED,        |
  | DISTRIBUTED, REVISED, MODIFIED, TRANSLATED, ABRIDGED, CONDENSED,       |
  | EXPANDED, COLLECTED, COMPILED, LINKED, RECAST, TRANSFORMED OR ADAPTED  |
  | WITHOUT THE PRIOR WRITTEN CONSENT OF NOVELL, INC.                      |
  |                                                                        |
  | ANY USE OR EXPLOITATION OF THIS WORK WITHOUT AUTHORIZATION COULD       |
  | SUBJECT THE PERPETRATOR TO CRIMINAL AND CIVIL LIABILITY.               |
  +************************************************************************+
*/
/*
  +************************************************************************+
  | Source module name:  vollib.h
  |
  |  $release$
  |  $modname$
  |  $version$
  |  $date$
  |  $nokeywords$
  +************************************************************************+
*/

#if !defined( VOLLIB_H )
#define VOLLIB_H

#if !defined(N_ARCH_16) && !defined(N_ARCH_32) && !defined(N_ARCH_64)
#define N_ARCH_32 1
#if !defined (N_PLAT_NLM)
#define N_PLAT_NLM 1
#endif
#endif

#include <ntypes.h>

/* API version numbers */
#define NWVL_MAJOR		1
#define NWVL_MINOR		0

/* Locking type */
#define NWVL_SHARED		1 /* read only */
#define NWVL_EXCLUSIVE	2 /* read/write */

/* Hard-coded limits */
#define NWVL_VOLNAME_BUFFER_SIZE	64 /* note that the max vol name size may be less than this */

/* Segment type definitions */ 
#define NWVL_FREE_SEGMENT		1
#define NWVL_INVALID_SEGMENT	2
#define NWVL_VALID_SEGMENT		3

/* Segment status codes  */
#define NWVL_SEGSTAT_READFAIL			0x00000001 /* Excessive read errors occurred */
#define NWVL_SEGSTAT_BUSY				0x00000002 /* Couldn't lock the partition */
#define NWVL_SEGSTAT_READERRORS			0x00000004 /* Read errors occurred */
#define NWVL_SEGSTAT_UNUSABLE			0x00000008 /* Volume definitions contradictory */
#define NWVL_SEGSTAT_INCONSISTENT		0x00000010 /* Some volume definitions don't make sense */
#define NWVL_SEGSTAT_DUPLICATE_VOL		0x00000020 /* Complete, but alternative volume */
#define NWVL_SEGSTAT_DUPLICATE_SEG		0x00000040 /* Duplicate segments for an existing volume */
#define NWVL_SEGSTAT_INCOMPLETE_VOL		0x00000080 /* Volume fragment; pieces missing */
#define NWVL_SEGSTAT_INCONSISTENT_VOL	0x00000100 /* Volume fragment; pieces don't fit */

/* Segment traversal flag definitions */
#define NWVL_TRAVERSE_PARTITION_LIST	0 
#define NWVL_TRAVERSE_VOLUME_LIST		1

/* Block size defines */
#define NWVL_4K_BLOCK			3
#define NWVL_8K_BLOCK			4
#define NWVL_16K_BLOCK			5
#define NWVL_32K_BLOCK			6
#define NWVL_64K_BLOCK			7

/* Volume flags */
#define NWVL_SYSTEM_VOLUME		0x01
#define	NWVL_READ_ONLY_VOLUME 	0x80

/* Root Entry Volume Flag bits */
#define NWVL_SUB_ALLOCATION_ENABLED_BIT		0x02	/* sub allocation units valid on this volume */
#define NWVL_FILE_COMPRESSION_ENABLED_BIT	0x04	/* file compression enabled on this volume */
#define NWVL_DATA_MIGRATION_ENABLED_BIT		0x08	/* data migration is allowed on this volume */

/* Warning codes */
#define NWVLWARN_VOLS_NEED_UPDATE		1   /* some volumes definitions found were inconsistent;
												if any volume transactions occur, these will
												be flushed to disk */

/* Error codes */
#define NWVLERR_INTERNAL				-1  /* an unexpected internal error occured */
#define NWVLERR_ALLOC					-2  /* could not allocate memory */
#define NWVLERR_NO_SEGMENTS				-3  /* no volume segments were found */
#define NWVLERR_INVALID_PARTITION		-4  /* an invalid partition object number was specified */
#define NWVLERR_INVALID_SEGMENT			-5  /* an invalid segment handle was specified */
#define NWVLERR_GET_INFO				-6  /* an attempt to get partition information failed */
#define NWVLERR_LONG_NAME				-7  /* the volume name cannot be more than 15 characters */
#define NWVLERR_SHORT_NAME				-8  /* the volume name must be at least 2 characters long */
#define NWVLERR_INVALID_LOGICAL			-9 /* Invalid logical partition number */
#define NWVLERR_UNDERSCORE_AT_BEGOREND	-10 /* the volume name cannot contain an underscore at the beginning or end */
#define NWVLERR_MULTIPLE_UNDERSCORE		-11 /* the volume name cannot contain multiple adjacent underscores */
#define NWVLERR_RESERVED_NAME			-12 /* the volume name is a reserved name */
#define NWVLERR_NAME_NOT_NEW			-13 /* the volume name is already being used */
#define NWVLERR_INVALID_OFFSET			-14 /* the offset specified cannot be used */
#define NWVLERR_NOT_FREE				-15 /* the specified segment must be a free segment */
#define NWVLERR_INVALID_BLOCKSHIFT		-16 /* invalid block shift factor, must be 3-7 */
#define NWVLERR_SECTORS_NOT_NORMALIZED	-17 /* number of sectors not a multiple of the block size */
#define NWVLERR_OVERLAP					-18 /* new volume to create crosses over other segment boundaries */
#define NWVLERR_INVALID_VOLUME			-19 /* an invalid volume handle was specified */
#define NWVLERR_VOLUME_BUSY				-20 /* part of the volume was locked */
#define NWVLERR_VOLUME_MOUNTED			-21 /* the volume was mounted and should not have been */
#define NWVLERR_IS_FREE					-22 /* the specified segment cannot be a free segment */
#define NWVLERR_FAIL_MOUNT				-23 /* cannot mount the volume */
#define NWVLERR_FAIL_DISMOUNT			-24 /* cannot dismount the volume */
#define NWVLERR_NO_PARTITIONS			-25 /* no NetWare partitions were found */
#define NWVLERR_CREATE_FAT				-26 /* was not able to create the volume FAT table */
#define NWVLERR_CREATE_DIR				-27 /* was not able to create the volume directory table */
#define NWVLERR_FAT_WRITE				-28 /* unable to write the vol fat/dir tables */
#define NWVLERR_DEFWRITEVERIFY			-29 /* unable to pre-read-before write the vol def table */
#define NWVLERR_DEFWRITE				-30 /* unable to write the vol def table */
#define NWVLERR_DEFREADAFTERWRITE		-31 /* unable to read-after-write verify the vol def table */
#define NWVLERR_DEFREALWRITE			-32 /* unable to write the vol def table */
#define NWVLERR_FAT_READ				-33 /* unable to read the vol fat tables */
#define NWVLERR_FAT_INCONSISTENT		-34 /* the fat length read is the incorrect size, or
												the primary and mirror fat don't match */
#define NWVLERR_FAT_ALLOC				-35 /* a free FAT block could not be found (the vol is full?) */
#define NWVLERR_PART_BUSY				-36 /* could not lock the partition */
#define NWVLERR_DIR_ALLOC				-37 /* a free Directory block could not be found (the vol is full?) */
#define NWVLERR_DIR_WRITE				-38 /* a directory block could not be written */
#define NWVLERR_DIR_REDIR				-39 /* an attempt to redirect a directory block write failed */
#define NWVLERR_DIR_INCONSISTENT		-40 /* the directory table is inconsistent */
#define NWVLERR_DIR_READ				-41 /* a directory block could not be read */
#define NWVLERR_VOLUME_NOT_MOUNTED		-42 /* the volume was not mounted and should have been */
#define NWVLERR_CANT_RESET_SUBALLOC		-43 /* Sub-allocation is on, can't turn it off */
#define NWVLERR_CANT_RESET_COMPR		-44 /* Compression is on, can't turn it off */
#define NWVLERR_NOT_ON_THIS_OS			-45 /* Call not supported on this OS. */
#define NWVLERR_CANT_GET_OS_VER			-46 /* Can't get OS server version. */
#define NWVLERR_CANT_GET_VOLINFO		-47 /* Can't get OS volume information. */
#define NWVLERR_NOT_NETWARE_VOL			-48 /* The volume specified is not a NetWare volume. */
#define NWVLERR_VOL_BUSY				-49 /* Can't lock the volume. */
#define NWVLERR_EXPAND_VOLTABLES		-50 /* Can't expand OS volume FAT and directory tables. */
#define NWVLERR_BUSY					-51 /* Didn't call NWVL_Start, or apis already in use. */
#define NWVLERR_NO_MORE_VOLUMES			-52 /* No more volumes found. */
#define NWVLERR_NO_MORE_SEGMENTS		-53 /* No more segments found. */
#define NWVLERR_DUPLICATE_SEGMENT		-54 /* The new segment matches an existing one. */
#define NWVLERR_CANT_REGISTER_EVENT		-55 /* Unable to register for an os event. */
#define NWVLERR_SECTORS_BELOW_MIN		-56 /* The number of sectors specified is below the minimum. */
#define NWVLERR_TOO_MANY_SEGMENTS		-57 /* No additional segments can be added on this partition. */
#define NWVLERR_TOO_MANY_VOLSEG			-58 /* No additional segments can be added to this volume. */
#define NWVLERR_INVALID_PARAM			-59 /* A parameter passed to a function was invalid. */
#define NWVLERR_API_VERSION				-60 /* The desired api version level is not supported. */
#define NWVLERR_ACCESS_NOT_ALLOWED		-61 /* Shared access doesn't allow the attempted operation */
#define NWVLERR_INVALID_CHARS			-62 /* The name contains one or more invalid characters.  Valid chars: "A..Z_0..9!@#$%&()" */
#define NWVLERR_INVALID_CALLER			-63 /* NWVL_Start not called, or caller id not found. */

/* Function prototypes */
#if defined(__cplusplus)
extern "C" {
#endif


/* Quad typedef */
typedef struct
{
	nuint32 low;		/* Least significant part	*/
	nuint32 high;		/* Most significant part	*/
} NWVL_QUAD;

/* Segment information typedef */
typedef struct
{
	/* General segment information */
	nuint32 type;  /* NWVL_FREE_SEGMENT, NWVL_VALID_SEGMENT, NWVL_INVALID_SEGMENT */
	nuint32 status; /* NWVL_SEGSTAT_READFAIL, etc. */

	/* Segment information associated with the partition. */
	NWVL_QUAD startingDiskSector;
	NWVL_QUAD sectorsInSegment;
	nuint32 mirrorObjectID;

	/* Segment information associated with the volume. */
	nwchar volumeName[NWVL_VOLNAME_BUFFER_SIZE];	/* unicode, undefined if free */
	nuint32 blockSize;			  		/* undefined if free; NWVL_4K_BLOCK, etc. */
	nuint32 numberOfSegments;			/* 1 - maximum sectors per volume; undefined if free */
	nuint32 mySegmentPosition;			/* 0 through numberOfSegments - 1; undefined if free */
	nuint32 volumeFlags;				/* undefined if free */
	NWVL_QUAD blocksInVolume;    		/* undefined if free */
	nuint32 volumeHandle;				/* undefined if free or invalid segment */
	nuint32 sectorSize;					/* bytes per sector in segment */
	nuint32 flags;						/* currently unused */
	nuint32 reserved[4];				/* undefined */
} NWVL_SegmentInfo;


N_EXTERN_LIBRARY(nint)
NWVL_Start
(
	nuint32 identity, 		/* nlm handle on the server */
	nuint32 majorVersion, 	/* NWVL_MAJOR */
	nuint32 minorVersion, 	/* NWVL_MINOR */
	nuint32 lockType,		/* NWVL_SHARED or NWVL_EXCLUSIVE */
	pnuint32 returnedCaller
);

N_EXTERN_LIBRARY(nint)
NWVL_End
(
	nuint32 caller
);

N_EXTERN_LIBRARY(nint)
NWVL_ScanForVolumes
(
	nuint32 caller
);

N_EXTERN_LIBRARY(nint)
NWVL_GetNextVolume
(
	nuint32 caller,
	pnuint32 lastVolumeHandle
);

N_EXTERN_LIBRARY(nint)
NWVL_GetVolumeFirstSegment
(
	nuint32 caller,
	nuint32 volumeHandle,
	pnuint32 firstSegmentHandle
);

N_EXTERN_LIBRARY(nint)
NWVL_GetMirrorObjectFirstSegment
(
	nuint32 caller,
	nuint32 mirrorObjectID,
	pnuint32 firstSegmentHandle
);

N_EXTERN_LIBRARY(nint)
NWVL_GetNextSegment
(
	nuint32 caller,
	nuint32 flag,
	pnuint32 lastSegmentHandle
);

N_EXTERN_LIBRARY(nint)
NWVL_GetSegmentInfo
(
	nuint32 caller,
	nuint32 segmentHandle,
	NWVL_SegmentInfo *segInfo
);

N_EXTERN_LIBRARY(nint)
NWVL_CreateANewVolume
(
	nuint32 caller,
	pnwchar volumeName, 			/* unicode */
	nuint32 blockSize, 				/* NWVL_4K_BLOCK, etc. */
	nuint32 volumeControlFlags, 	/* NWVL_SUB_ALLOCATION_ENABLED_BIT, etc. */
	nuint32 dsObjectID,
	nuint32 mirrorObjectID,
	NWVL_QUAD startingSector,
	NWVL_QUAD sectorsInSegment,
	pnuint32 returnVolumeHandle
);

N_EXTERN_LIBRARY(nint)
NWVL_EnlargeVolume
(
	nuint32 caller,
	nuint32 volumeHandle,
	nuint32 mirrorObjectID,
	NWVL_QUAD startingSector,
	NWVL_QUAD sectorsInSegment
);

N_EXTERN_LIBRARY(nint)
NWVL_DeleteAVolume
(
	nuint32 caller,
	nuint32 volumeHandle
);

N_EXTERN_LIBRARY(nint)
NWVL_ChangeVolumeName
(
	nuint32 caller,
	nuint32 volumeHandle,
	pnwchar newVolName			/* unicode */
);

N_EXTERN_LIBRARY(nint)
NWVL_ChangeSegmentName
(
	nuint32 caller,
	nuint32 segmentHandle,
	pnwchar newName				/* unicode */
);
	
N_EXTERN_LIBRARY(nint)
NWVL_DeleteSegment
(
	nuint32 caller,
	nuint32 segmentHandle
);

N_EXTERN_LIBRARY(nint)
NWVL_IsVolumeMounted
(
	nuint32 caller,
	pnwchar volumeName,			/* unicode */
	pnuint32 mountedFlag
);

N_EXTERN_LIBRARY(nint)
NWVL_MountVolume
(
	nuint32 caller,
	pnwchar volumeName			/* unicode */
);

N_EXTERN_LIBRARY(nint)
NWVL_DismountVolume
(
	nuint32 caller,
	pnwchar volumeName			/* unicode */
);

N_EXTERN_LIBRARY(nint)
NWVL_SetOrGetVolumeControlInfo
(
	nuint32 caller,
	nuint32 volumeHandle,
	nuint32 action, 			/* 0: get, 1: set */
	pnuint32 controlFlags,		/* passed or returned depending on the action - can be NULL */
	pnuint32 dsID  				/* passed or returned depending on the action - can be NULL */
);

N_EXTERN_LIBRARY(nint)
NWVL_ValidateVolumeName
(
	nuint32 caller,
	pnwchar volName /* unicode */
);

N_EXTERN_LIBRARY(nint)
NWVL_GetDefaultBlockSizeAndControlFlags
(
	nuint32 caller,
	NWVL_QUAD numberOfSectors,
	pnuint32 blockSize,
	pnuint32 controlFlags
);

N_EXTERN_LIBRARY(nint)
NWVL_GetSupportedLimits
(
	nuint32 caller,
	pnuint32 maximumVolumeNameSize, 	/* in unicode characters, including null character */
	pnuint32 quadBits, 					/* bits in a quad */
	pnuint32 maximumSegmentsPerVolume	/* max segments allowed in a volume */
);
	
N_EXTERN_LIBRARY(nint)
NWVL_GetMirrorObjectSupportedLimits
(
	nuint32 caller,
	nuint32 mirrorObjectID,
	pnuint32 maximumSegmentsPerPartition,  /* max segments allowed per logical partition */
	pnuint32 minimumSectorsPerSegment,		/* min sectors allowed per segment */
	pnuint32 sectorSize						/* bytes per sector */
);

N_EXTERN_LIBRARY(nint)
NWVL_ConvertMirrorObjectToLogicalPartition
(
	nuint32 caller,
	nuint32 mirrorObjectID,
	pnuint32 logicalPartitionNumber
);

N_EXTERN_LIBRARY(nint)
NWVL_ConvertLogicalPartitionToMirrorObject
(
	nuint32 caller,
	nuint32 logicalPartitionNumber,
	pnuint32 mirrorObjectID
);

N_EXTERN_LIBRARY(nint)
NWVL_LocalToUnicode
(
	nuint32 caller,
	pnchar localChars,
	nuint32 maxUniChars,
	pnwchar uniChars 			/* unicode */
);
	
N_EXTERN_LIBRARY(nint)
NWVL_UnicodeToLocal
(
	nuint32 caller,
	pnwchar uniChars, /* unicode */
	nuint32 maxLocalChars,
	pnchar localChars
);

N_EXTERN_LIBRARY(nint)
NWVL_Nuint32ToQuad
(
	nuint32 caller,
	nuint32 value,
	NWVL_QUAD *quadValue
);

N_EXTERN_LIBRARY(nint)
NWVL_QuadToNuint32
(
	nuint32 caller,
	NWVL_QUAD quadValue,
	pnuint32 value
);


#if defined(__cplusplus)
}
#endif

#endif /* !defined(VOLLIB_H) */

